/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "config.h"

#include "addmedium.h"

#include "mediamodel.h"
#include "medium.h"
#include "sqlconstraint.h"
#include "msqldatabase.h"

#include <QStringList>
#include <QSortFilterProxyModel>
#include <QSqlQuery>

AddMedium::AddMedium ( QWidget* parent, Qt::WindowFlags flags )
	: MDialog ( parent, flags )
{
	init();
}

AddMedium::AddMedium( MediaModel* mediaModel, QWidget* parent, Qt::WindowFlags flags)
	: MDialog(parent, flags)
{
	init(mediaModel);
}

AddMedium::~AddMedium()
{
	this->close();
	mediumView->setModel(0);
// 	delete searchModel;
}

Medium* AddMedium::selectedMedium()
{
	return myMedium;
}

void AddMedium::search()
{
	if(!searchModel)
		return;
	
	searchModel->clear();
	
	QList<SqlConstraint> constraints;
	if(!titleEdit->text().isEmpty()){
		SqlConstraint constraint;
		constraint.setConstraint("title LIKE ?");
		constraint.addBindValue(QString("%%1%" ).arg ( titleEdit->text()) );
		constraints << constraint;
	}
	
	QList<SearchPlugin*> searchPlugins;
	searchModel->load(constraints, searchPlugins, myUsersId);
}

void AddMedium::newMedium()
{

}

void AddMedium::addMedium()
{
// 	if(!targetModel){
// 		emit error(tr("No target model specified!"));
// 		return;
// 	}
	Medium *medium = searchModel->mediumFromIndex(mediumIndex);
	if(!medium){
		emit error(tr("Please select a medium!"));
		return;
	}
	
	// remove the medium from the search model,
	// because we destroy the search model including all childs
	// in our desctructor
	if(!searchModel->removeMedium(medium)){
		emit error(tr("Could not remove medium from search model!"));
		return;
	}
	myMedium = medium;
	if(targetModel)
		targetModel->addMedium(medium);
	
	accept();
}

void AddMedium::indexChanged(QModelIndex index)
{
	mediumIndex = proxyModel->mapToSource(index);
}


void AddMedium::init(MediaModel *targetModel)
{
	myUsersId = 0;
	myMedium = 0;
	
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if(db.isOpen()){
		QSqlQuery query;
		query.prepare("SELECT id FROM users WHERE userName=:userName");
		query.bindValue(":userName", db.userName());
		query.exec();
		if(query.first())
			myUsersId = query.record().value("id").toUInt();
		if(closeDB)
			db.close();
	}
	
	setupUi(this);
	this->targetModel = targetModel;
	if(!targetModel){
		searchModel = new MediaModel(0, this);
	}
	else{
		searchModel = targetModel->createNewModel();;
	}
	connect(searchModel, SIGNAL(error(QString)), this, SLOT(errorByPass(QString)));
  connect(searchModel, SIGNAL(warning(QString)), this, SLOT(warningByPass(QString)));
  connect(searchModel, SIGNAL(information(QString)), this, SLOT(informationByPass(QString)));
	
	proxyModel = new QSortFilterProxyModel(this);
	proxyModel->setSourceModel(searchModel);
	// apply changes off the original model to the proxy model
  proxyModel->setDynamicSortFilter ( true );
//   proxyModel->sort ( 0 );
//   proxyModel->setFilterKeyColumn ( 0 );
//   proxyModel->setFilterCaseSensitivity ( Qt::CaseInsensitive );
	mediumView->setModel(proxyModel);
	mediumView->setSortingEnabled(true);
	searchModel->configureView(mediumView);
	
	connect(searchButton, SIGNAL(clicked()), this, SLOT(search()));
	searchButton->setIcon(QIcon(":/icons/filefind.png"));
	connect(mediumView, SIGNAL(clicked(QModelIndex)), this, SLOT(indexChanged(QModelIndex)));
	connect(addButton, SIGNAL(clicked()), this, SLOT(addMedium()));
	connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));

#if QT_VERSION_MINOR < 6
	newButton->setIcon ( QIcon ( ":/icons/filenew.png" ) );
#else
	newButton->setIcon ( QIcon::fromTheme ( "document-new",
	                          QIcon ( ":/icons/filenew.png" ) ) );
#endif
	
	setHelp("keyword:Add Medium", helpButton);
	
	search();
}

#include "addmedium.moc"
