/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "dbbackupdialog.h"

#include "dbbackup.h"
#include <mpushbutton.h>

#include <QVBoxLayout>
#include <QHBoxLayout>

DBBackupDialog::DBBackupDialog(QWidget* parent, Qt::WindowFlags flags)
	: MDialog(parent, flags)
{
	inProgress = false;
	
	setObjectName("DBBackupDialog");
	setWindowTitle(tr("Database Backups"));
	
	QVBoxLayout *ml = new QVBoxLayout(this);
	setLayout(ml);
	
	dbBackup = new DBBackup(0, this);
	ml->addWidget(dbBackup);
	connectByPass(dbBackup);
	connect(dbBackup, SIGNAL(exportStarted()), this, SLOT(exportStarted()));
	connect(dbBackup, SIGNAL(exportFinished(bool)), this, SLOT(exportFinished(bool)));
	
	QHBoxLayout *bl = new QHBoxLayout();
	ml->addLayout(bl);
	
	MPushButton *button = new MPushButton(this);
	bl->addWidget(button);
	setHelp("keyword:DBBackup", button);
	
	bl->addStretch();
	
	exportButton = new MPushButton(tr("Export"), this);
	bl->addWidget(exportButton);
	connect(exportButton, SIGNAL(clicked()), this, SLOT(finish()));
	
	closeButton = new MPushButton(this);
	bl->addWidget(closeButton);
	connect(closeButton, SIGNAL(clicked()), this, SLOT(abort()));
	exportFinished(false);
}

void DBBackupDialog::finish()
{
	dbBackup->finish();
}

void DBBackupDialog::abort()
{
	if(inProgress){
		dbBackup->abort();
	}
	else{
		if(doneSuccessfully){
			accept();
		}
		else{
			reject();
		}
	}
}

void DBBackupDialog::exportStarted()
{
	inProgress = true;
	exportButton->setEnabled(false);
	closeButton->setText(tr("Cancel"));
}

void DBBackupDialog::exportFinished(bool result)
{
	inProgress = false;
	exportButton->setEnabled(true);
	closeButton->setText(tr("Close"));
	doneSuccessfully = result;
}

#include "dbbackupdialog.moc"
