/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "editmanufacturersdetailswidget.h"

#include "msqldatabase.h"

#include <QSqlQueryModel>
#include <QSqlQuery>
#include <QSqlError>
#include <QSqlRecord>
#include <QCompleter>

EditManufacturersDetailsWidget::EditManufacturersDetailsWidget(QSqlQueryModel* model, QWidget* parent, Qt::WindowFlags flags)
	: DetailsWidget(parent, flags)
{
	setupUi ( this );
		
	this->model = model;
	
	connect(websiteEdit, SIGNAL(editingFinished()), this, SLOT(websiteFinished()));
	
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if ( db.isOpen() ) {
    QSqlQueryModel *complCity = new QSqlQueryModel ( this );
    complCity->setQuery ( "SELECT city FROM manufacturers ORDER BY city" );
    if ( !complCity->lastError().isValid() ) {
      QCompleter *completer = new QCompleter ( complCity, this );
      completer->setCaseSensitivity ( Qt::CaseInsensitive );
      completer->setModelSorting ( QCompleter::CaseInsensitivelySortedModel );
      cityEdit->setCompleter ( completer );
    }

    QSqlQueryModel *complCountry = new QSqlQueryModel ( this );
    complCountry->setQuery ( "SELECT country FROM manufacturers GROUP BY country" );
    if ( !complCountry->lastError().isValid() ) {
      QCompleter *completer = new QCompleter ( complCountry, this );
      completer->setCaseSensitivity ( Qt::CaseInsensitive );
      completer->setModelSorting ( QCompleter::CaseInsensitivelySortedModel );
      countryEdit->setCompleter ( completer );
    }
		if(closeDB)
			db.close();
  }
}

QString EditManufacturersDetailsWidget::name()
{
	return nameEdit->text();
}

bool EditManufacturersDetailsWidget::store()
{
	if ( nameEdit->text().isEmpty() ) {
    emit error ( tr ( "An empty name is not allowed!" ) );
    return false;
  }

	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB){
		if ( !db.open() ) {
			emit error ( tr ( "Could not connect to database" ) );
			return false;
		}
	}

	QSqlQuery query;
	if(curIndex.isValid()){
		// a present manufacturer was edited --> we must do a update
		query.prepare("UPDATE manufacturers SET name=:name, street=:street, number=:number,"
																	" city=:city, postalCode=:postalCode, postOfficeBox=:postOfficeBox, country=:country, website=:website,"
																	" email=:email, phone=:phone, fax=:fax"
																	" WHERE id=:id");
		query.bindValue(":id", model->record(curIndex.row()).value("id").toUInt());
	}
  else{
		query.prepare("INSERT INTO manufacturers (name, street, number, city, postalCode, postOfficeBox,"
									"  country, website, email, phone, fax)"
									" VALUES(:name, :street, :number, :city, :postalCode, :postOfficeBox,"
									"  :country, :website, :email, :phone, :fax)");
	}
	query.bindValue(":name", nameEdit->text());
	query.bindValue(":street", streetEdit->text());
	query.bindValue(":number", numberEdit->text());
	query.bindValue(":city", cityEdit->text());
	query.bindValue(":postalCode", postalCodeEdit->value());
	query.bindValue(":postOfficeBox", postOfficeBoxEdit->value());
	query.bindValue(":country", countryEdit->text());
	query.bindValue(":website", websiteEdit->text());
	query.bindValue(":email", emailEdit->text());
	query.bindValue(":phone", phoneEdit->text());
	query.bindValue(":fax", faxEdit->text());
	if ( !query.exec()){
		emit error ( tr ( "Could not insert or update manufacturer %1\nQuery: %2\nError: %3" )
			.arg ( nameEdit->text() )
			.arg(query.lastQuery())
			.arg(query.lastError().text()) );
		if(closeDB)
			db.close();
		return false;
	}
		
  // reload model
  if ( model ) {
    model->setQuery ( model->query().lastQuery() );
  }
  if(closeDB)
		db.close();
	
	return true;
}

void EditManufacturersDetailsWidget::clear()
{
	// setup the details widget
	nameEdit->setText ( "" );
  streetEdit->setText ( "" );
  numberEdit->setText ( "" );
  cityEdit->setText ( "" );
  postalCodeEdit->setValue ( 0 );
  countryEdit->setText ( "" );
  websiteEdit->setText ( "" );
  emailEdit->setText ( "" );
  phoneEdit->setText ( "" );
  faxEdit->setText ( "" );
}

void EditManufacturersDetailsWidget::setItem(QModelIndex index)
{
	if(!model)
		return;
	
	if(!index.isValid())
		return;
	
	curIndex = index;
	
	clear();
	
	QSqlRecord record = model->record(curIndex.row());
	nameEdit->setText(record.value("name").toString());
	streetEdit->setText(record.value("street").toString());
	numberEdit->setText(record.value("number").toString());
	cityEdit->setText(record.value("city").toString());
	postalCodeEdit->setValue(record.value("postalCode").toInt());
	countryEdit->setText(record.value("country").toString());
	websiteEdit->setText(record.value("website").toString());
	emailEdit->setText(record.value("email").toString());
	phoneEdit->setText(record.value("phone").toString());
	faxEdit->setText(record.value("fax").toString());
}

void EditManufacturersDetailsWidget::setEnabled(bool show, QString defaultString)
{
	clear();
	if(!defaultString.isEmpty())
		nameEdit->setText(defaultString);
	MWidget::setEnabled(show);
	nameEdit->setFocus();
}

void EditManufacturersDetailsWidget::websiteFinished()
{
	if(!websiteEdit->text().contains(QRegExp("^https?://")))
		websiteEdit->setText("http://"+ websiteEdit->text());
}

#include "editmanufacturersdetailswidget.moc"
