/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef EDITMEDIUM_H
#define EDITMEDIUM_H

#include "libmacro.h"

#include "mdialog.h"
#include "ui_editmedium.h"

class Medium;
class MediaModel;

/**
	* @todo implement mediumChange and needSave functionality
	* @todo but think about this: The Medium*Widget's must emit if sum data is changed and not submitted
	* @todo see #112
	*/
class EXPORT_DLL EditMedium : public MDialog, protected Ui::EditMedium
{
	Q_OBJECT
	public:
		enum EditMode { Edit, Add, New };
		EditMedium(QWidget *parent = 0, Qt::WindowFlags flags = 0);
		/**
			* if called, the window title gets changed to "Add Medium to Database"
			*/
		void setAddMode(Medium *medium, bool newTransaction = true);
		/**
			* if called, the window title gets changed to "Edit Medium"
			* this is the standard
			*/
		void setEditMode(Medium *medium, bool newTransaction = true);
		void setNewMode(MediaModel *model, Medium *medium, bool newTransaction = true);

	public slots:
		void save();
		
	protected:
		/**
			* do all customized loading stuff in here
			*/
		virtual void loadData(Medium *medium){};
		/**
			* do all customized saving functions here
			* @return true if a Medium::store call is enough to store all changes!
			* @return All other cases Medium cannot be saved correctly
			*/
		virtual bool prepareSave(Medium *medium){ return true; };
		void insertWidget(QWidget *newWidget, QWidget *intoWidget, QWidget *afterWidget = 0);
		void insertLayout(QLayout *newLayout, QWidget *intoWidget, QWidget *afterWidget = 0);
		/**
			* use this class to create a label with configured size and sizepolicy
			* the text of label is set to @p text
			*/
		QLabel *prepareLabel(QString text);
		QWidget *tabOrder(QWidget* lastWidget, QLayout* childlayout);

	private slots:
		void cancel();
		void mediumChanged();
		
	private:
		/**
			* this function loads the data of medium into the standard widgets
			* afterwards it calls loadData to allow further custom loading
			*/
		void setMedium(Medium* medium, bool newTransaction);
		/**
			* with this function you can insert some new widgets
			* it tries to insert @p newWidget into the layout of @p intoWidget after @p afterWidget
			* If @p afterWidget = 0, the new Widget is appended at the end
			* @todo insertItem seems to come public function in Qt5, review code than!
			* @todo change taborder 
			*/
		void insertItem(QLayoutItem *newItem, QWidget *intoWidget, QWidget *afterWidget = 0);

		Medium *medium;
		MediaModel *model;
		EditMode editMode;
		bool needSave;
		bool startTransaction;
};

#endif // EDITMEDIUM_H
