/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "maction.h"

#include <QSettings>
#include <QIcon>

MAction::MAction ( QObject* object ) : QAction ( object )
{
	init();
}

MAction::MAction ( const QString& text, QObject* parent ) : QAction ( text, parent )
{
	init();
	setText(text);
}

MAction::MAction ( const QIcon& icon, const QString& text, QObject* parent ) : QAction ( icon, text, parent )
{
	init();
	setText(text);
}

MAction::~MAction()
{
	delete myActionWithText;
}

QAction* MAction::actionWithText()
{
	if(!myActionWithText){
		myActionWithText = new QAction(icon(), myText, this);
		connect(myActionWithText, SIGNAL(triggered()), this, SLOT(triggeredByPass()));
	}
	return myActionWithText;	
}

void MAction::setText(const QString& text)
{
	myText = text;
	if(toolTip().isEmpty())
		setToolTip(myText);
	
	if(iconsAndText || icon().isNull())
		QAction::setText(myText);
	else
		QAction::setText("");

	if(myActionWithText)
		myActionWithText->setText(myText);
}

void MAction::setEnabled ( bool status )
{
	if(myActionWithText)
		myActionWithText->setEnabled(status);
	QAction::setEnabled(status);
}

void MAction::setDisabled ( bool status )
{
	setEnabled(!status);
}

void MAction::triggeredByPass()
{
	emit triggered();
}

void MAction::init()
{
	myActionWithText = 0;

	QSettings qSettings;
  iconsAndText = qSettings.value ( "iconsAndText", false ).toBool();
}

#include "maction.moc"
