/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediumboughtpricewidget.h"

#include "medium.h"
#include "msqldatabase.h"

#include <QSqlQueryModel>
#include <QSqlRecord>
#include <QCompleter>

MediumBoughtPriceWidget::MediumBoughtPriceWidget(Medium* medium, QWidget* parent, Qt::WindowFlags flags)
	: MWidget(parent, flags)
{
	init();
	setMedium(medium);
}

MediumBoughtPriceWidget::MediumBoughtPriceWidget(QWidget* parent, Qt::WindowFlags flags)
	: MWidget(parent, flags)
{
	init();
	setMedium(0);
}

void MediumBoughtPriceWidget::setMedium(Medium* medium)
{
	if(this->medium)
		medium->disconnect(this);
	this->medium = medium;
	
// 	if(medium)
// 		connect(medium, SIGNAL(changed()), this, SLOT(setPriceFromMedium()));
	
	bool enable = true;
	if(!medium)
		enable = false;
	setEnabled(enable);
	setPriceFromMedium();
}

void MediumBoughtPriceWidget::setPriceFromMedium()
{
	if(medium){
		priceEdit->setValue(medium->boughtPrice());
		currencyEdit->setText(mapCurrencies.value(medium->boughtPriceCurrency()));
	}
}

void MediumBoughtPriceWidget::setPrice()
{
	if(!medium)
		return;
	
	medium->setBoughtPrice(priceEdit->value());
}

void MediumBoughtPriceWidget::setCurrency()
{
	if(!medium)
		return;
	
	medium->setBoughtPriceCurrency(mapCurrencies.key(currencyEdit->text()));
}

void MediumBoughtPriceWidget::init()
{
	this->medium = 0;
	complModel = 0;
	
	setupUi(this);
	priceLabel->setText(tr("Bought Price:"));
	connect(priceEdit, SIGNAL(editingFinished()), this, SLOT(setPrice()));
	connect(currencyEdit, SIGNAL(editingFinished()), this, SLOT(setCurrency()));
	loadCompleter();
}

void MediumBoughtPriceWidget::loadCompleter()
{
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		if(!db.open())
			return;
		
	QString query = "SELECT id, shortName FROM currencies ORDER BY shortName";
	if(!complModel){
		complModel = new QSqlQueryModel ( this );
		QCompleter *completer = new QCompleter ( complModel, this );
		completer->setCaseSensitivity ( Qt::CaseInsensitive );
		completer->setModelSorting ( QCompleter::CaseInsensitivelySortedModel );
		currencyEdit->setCompleter ( completer );
	}
	complModel->setQuery ( query );
	if(closeDB)
		db.close();
	
	for(int i = 0; i < complModel->rowCount(); i++){
		QSqlRecord result = complModel->record(i);
		mapCurrencies.insert(result.value("id").toUInt(), result.value("shortName").toString());
	}
}

#include "mediumboughtpricewidget.moc"
