/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediumkeywordwidget.h"
#include "mediumkeywordwidget.moc"

#include "msqldatabase.h"
#include "medium.h"

#include <QSqlQueryModel>
#include <QCompleter>
#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlError>
#include <QStringListModel>

MediumKeywordWidget::MediumKeywordWidget(QWidget* parent, Qt::WindowFlags flags)
	: MediumEditWidget(parent, flags)
{
	completerModel = new QSqlQueryModel(this);
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if(db.isOpen()){
		completerModel->setQuery("SELECT * FROM keywords ORDER BY keyword");
		if(closeDB)
			db.close();
	}
	setupUi(this);

	QCompleter *completer = new QCompleter(completerModel, this);
	completer->setCaseSensitivity ( Qt::CaseInsensitive );
	completer->setModelSorting ( QCompleter::CaseInsensitivelySortedModel );
	completer->setCompletionColumn(completerModel->record().indexOf("keyword"));
	keywordEdit->setCompleter ( completer );

	setEnabled(false);

	connect(keywordList, SIGNAL(clicked(QModelIndex)), this, SLOT(selectedKeyword(QModelIndex)));
	connect(addButton, SIGNAL(clicked()), this, SLOT(addKeyword()));
	connect(removeButton, SIGNAL(clicked()), this, SLOT(removeKeyword()));
}

bool MediumKeywordWidget::store(bool useExistingTransaction)
{
	// we have nothing to do here, because Medium handle all stuff
	if(!medium)
		return false;

	return true;
}

void MediumKeywordWidget::setMedium(Medium* newMedium)
{
	medium = newMedium;
	if(!medium){
		setEnabled(false);
		return;
	}
	renewModel();
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	completerModel->setQuery(completerModel->query().lastQuery());
	if(closeDB)
		db.close();
	
	setEnabled(true);
}

void MediumKeywordWidget::addKeyword()
{
	if(!medium)
		return;

	if(keywordEdit->text().isEmpty())
		return;
	
	medium->addKeyword(keywordEdit->text());
	renewModel();
	keywordEdit->setText(QString());
}

void MediumKeywordWidget::removeKeyword()
{
	if(!medium)
		return;

	QString keyword = keywordList->model()->data(index).toString();
	if(!medium->removeKeyword(keyword))
		emit warning(tr("Could not remove keyword %1!").arg(keyword));
	renewModel();
}

void MediumKeywordWidget::selectedKeyword(QModelIndex index)
{
	this->index = index;

	removeButton->setEnabled(false);
	if(!index.isValid())
		return;
	removeButton->setEnabled(true);
}

void MediumKeywordWidget::renewModel()
{
	selectedKeyword(QModelIndex());
	QStringListModel *model = qobject_cast<QStringListModel*>(keywordList->model());
	keywordList->setModel(0);
	if(model){
		delete model;
		model = 0;
	}

	if(!medium)
		return;
	
	model = medium->keywords();
	keywordList->setModel(model);
}
