/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediumtitlewidget.h"

#include "medium.h"
#include "msqldatabase.h"

#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QSqlQueryModel>
#include <QCompleter>

MediumTitleWidget::MediumTitleWidget(Medium* medium, QWidget* parent, Qt::WindowFlags flags)
	: MWidget(parent, flags)
{
	setupUi();
	setMedium(medium);
}

MediumTitleWidget::MediumTitleWidget(QWidget* parent, Qt::WindowFlags flags)
	: MWidget(parent, flags)
{
	setupUi();
	setMedium(0);
}

void MediumTitleWidget::setMedium(Medium* medium)
{
	if(this->medium)
		medium->disconnect(this);
	this->medium = medium;
	
// 	if(medium)
// 		connect(medium, SIGNAL(changed()), this, SLOT(setTitleFromMedium()));
	
	bool enable = true;
	if(!medium)
		enable = false;
	setEnabled(enable);
	setTitleFromMedium();
}

void MediumTitleWidget::setTitle()
{
	if(!medium)
		return;
	
	medium->setTitle(titleEdit->text());
}

void MediumTitleWidget::setTitleFromMedium()
{
	if(medium)
		titleEdit->setText(medium->title());
}

void MediumTitleWidget::checkLength()
{
	if(titleEdit->text().length() <= 120)
		return;
	
	titleEdit->setText(titleEdit->text().left(120));
	emit information(tr("Title can not be longer than 120 characters"));
}

void MediumTitleWidget::setupUi()
{
	this->medium = 0;
	complModel = 0;
	
	QHBoxLayout *mainLayout = new QHBoxLayout(this);
	setLayout(mainLayout);
	mainLayout->setMargin(0);
	
	QLabel *label = new QLabel(tr("Title:"), this);
	mainLayout->addWidget(label);
	label->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
	label->setMinimumSize(150, 23);
	
	titleEdit = new QLineEdit(this);
	mainLayout->addWidget(titleEdit);
	connect(titleEdit, SIGNAL(editingFinished()), this, SLOT(setTitle()));
	connect(titleEdit, SIGNAL(textChanged(QString)), this, SLOT(checkLength()));
	titleEdit->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
	titleEdit->setMaximumHeight(30);
	
	loadCompleter();
}

void MediumTitleWidget::loadCompleter()
{
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		if(!db.open())
			return;
		
	QString query = "SELECT title FROM materials ORDER BY name LIMIT 500";
	if(!complModel){
		complModel = new QSqlQueryModel ( this );
		QCompleter *completer = new QCompleter ( complModel, this );
		completer->setCaseSensitivity ( Qt::CaseInsensitive );
		completer->setModelSorting ( QCompleter::CaseInsensitivelySortedModel );
		titleEdit->setCompleter ( completer );
	}
	complModel->setQuery ( query );
	if(closeDB)
		db.close();
}

#include "mediumtitlewidget.moc"
