/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "config.h"

#include "minvalidator.h"

#include "minvalidatorplugin.h"
#include "modulelistwidgetitem.h"

#include <QPluginLoader>
#include <QDir>

MINValidator::MINValidator(QObject* parent): QValidator(parent)
{
  //load and test plugin
  QDir dir ( QString(MINPLUGINDIR) );
	
  QStringList files = dir.entryList ( QDir::Files );

  foreach ( QString file, files ) {
		file = dir.absolutePath() +"/"+ file;
		ModuleListWidgetItem *item = new ModuleListWidgetItem();
		pluginProtocol.append ( item );
		item->setFile ( file );
		
    QPluginLoader loader ( file );
		QObject *plugin = loader.instance();
		if(!plugin){
			item->setStatus ( false );
			item->setErrorMessage ( tr ( "While loading %1 an error occured:\n%2 %1" ).arg ( file ).arg(loader.errorString()) );
		}
		else{
			MINValidatorPlugin *newPlugin = qobject_cast<MINValidatorPlugin *> ( plugin );
			if(!newPlugin){
				item->setStatus ( false );
				item->setErrorMessage ( tr ( "Plugin is not a MINValidator-plugin:\n %1" ).arg ( file ) );
			}
			else{
				item->setStatus(true);
				item->setName(newPlugin->type());
				item->setDescription(newPlugin->description());
				QString type = newPlugin->type();
				if(!types.contains(type))
					types << type;
				plugins << newPlugin;
			}
		}
  }
}

MINValidator::~MINValidator()
{
// 	unloadPlugins();
// 	qDeleteAll(pluginProtocol);
}

QList< ModuleListWidgetItem* > MINValidator::loadProtocol()
{
	return pluginProtocol;
}

QStringList MINValidator::supportedTypes() const
{
	return types;
}

QString MINValidator::descriptionToType(QString type) const
{
	foreach(MINValidatorPlugin *plugin, plugins){
		if(plugin->type() == type)
			return plugin->description();
	}
	return QString();
}

bool MINValidator::isSupported(QString type) const
{
	foreach(MINValidatorPlugin *plugin, plugins)
		if(plugin->type() == type)
			return true;
	return false;
}

void MINValidator::fixup(QString& input) const
{
	if(possibleTypes(input).size() < 1)
		return;
  
	QString result = validate(input, QString());
	if(!result.isEmpty())
		input = result;
}

QValidator::State MINValidator::validate(QString& input, int& pos) const
{
	if(isValid(input))
		return QValidator::Acceptable;
	
	if(possibleTypes(input).size() > 0)
		return QValidator::Intermediate;
	
	return QValidator::Invalid;
}

QString MINValidator::type(QString min) const
{
	foreach(MINValidatorPlugin *plugin, plugins){
		if(plugin->isType(min))
			return plugin->type();
	}
	return QString();
}

QStringList MINValidator::possibleTypes(QString min) const
{
	QStringList result;
	foreach(MINValidatorPlugin *plugin, plugins){
		if(plugin->mayType(min))
			result << plugin->type();
	}
	return result;
}

bool MINValidator::isValid(QString min, QString type) const
{
	if(type.isEmpty())
		type = this->type(min);
	if(type.isEmpty())
		return false;
	
	foreach(MINValidatorPlugin *plugin, plugins){
		if(plugin->type() == type){
			if(plugin->isValid(min))
				return true;
			return false;
		}
	}
	return false;
}

QString MINValidator::validate(QString min, QString type) const
{
	if(type.isEmpty()){
		// try to guess type
		QStringList mayTypes = possibleTypes(min);
		if(mayTypes.size() != 1)
			return QString();
		type = mayTypes[0];
	}

	foreach(MINValidatorPlugin *plugin, plugins){
		if(plugin->type() == type)
			return plugin->validate(min);
	}
	return QString();
}

QString MINValidator::convert(QString min, QString toType, QString fromType) const
{
	if(toType.isEmpty())
		return QString();
	
	if(fromType.isEmpty())
		fromType = this->type(min);
	if(fromType.isEmpty())
		return QString();
	
	foreach(MINValidatorPlugin *plugin, plugins){
		if(plugin->canConvert(toType)){
			if((fromType.isEmpty()) || (plugin->type() == fromType)){
				QString newMin = plugin->convert(min, toType);
				if(newMin.isEmpty())
					continue;
				newMin = validate(newMin, toType);
				if(newMin.isEmpty())
					continue;
				if(isValid(newMin, toType))
					return newMin;
			}
		}
	}
	return QString();
}

QString MINValidator::strip(QString min, QString type) const
{
	if(type.isEmpty())
		type = this->type(min);
	if(type.isEmpty())
		return QString();
	
	foreach(MINValidatorPlugin *plugin, plugins){
		if(plugin->type() == type)
			return plugin->strip(min);
	}
	return QString();
}

void MINValidator::unloadPlugins ( )
{
	foreach(MINValidatorPlugin *object, plugins){
		if ( object ) {
			plugins.removeAll ( object );
			foreach ( ModuleListWidgetItem *item, pluginProtocol ) {
				if ( item->name() == object->type() ) {
					pluginProtocol.removeAll ( item );
					delete item;
					break;
				}
			}
			
			delete object;
		}
	}
}

#include "minvalidator.moc"
