/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mobject.h"

#include <QSettings>

MObject::MObject ( QObject *parent ) : QObject ( parent )
{
  QSettings qSettings;
  iconsAndText = qSettings.value ( "iconsAndText" ).toBool();
	alternatingRows = qSettings.value("alternatingRowColors").toBool();
}

MObject::~MObject()
{

}

void MObject::connectByPass ( QObject *object )
{
  connect ( object, SIGNAL ( error ( const QString& ) ), this, SLOT ( errorByPass ( const QString& ) ) );
  connect ( object, SIGNAL ( warning ( const QString& ) ), this, SLOT ( warningByPass ( const QString& ) ) );
  connect ( object, SIGNAL ( information ( const QString& ) ), this, SLOT ( informationByPass ( const QString& ) ) );

  connect ( object, SIGNAL ( addMenu ( QMenu*, const QString& ) ), this, SLOT ( addMenuByPass ( QMenu*, const QString& ) ) );
  connect ( object, SIGNAL ( delMenu ( QMenu* ) ), this, SLOT ( delMenuByPass ( QMenu* ) ) );
  connect ( object, SIGNAL ( addAction ( QAction *, const QString& ) ), this, SLOT ( addActionByPass ( QAction*, const QString& ) ) );
  connect ( object, SIGNAL ( delAction ( QAction* ) ), this, SLOT ( delActionByPass ( QAction* ) ) );
  connect ( object, SIGNAL ( addToolbar ( QToolBar * ) ), this, SLOT ( addToolbarByPass ( QToolBar* ) ) );
  connect ( object, SIGNAL ( delToolbar ( QToolBar* ) ), this, SLOT ( delToolbarByPass ( QToolBar* ) ) );

  connect ( object, SIGNAL ( addNewDockWidget ( QDockWidget*, const Qt::DockWidgetArea& ) ), this, SLOT ( addNewDockWidgetByPass ( QDockWidget*, const Qt::DockWidgetArea& ) ) );
  connect ( object, SIGNAL ( removeDockWidget ( QDockWidget* ) ), this, SLOT ( removeDockWidgetByPass ( QDockWidget* ) ) );
  connect ( object, SIGNAL ( setCentralWidget ( QWidget* ) ), this, SLOT ( setCentralWidgetByPass ( QWidget* ) ) );
  connect ( object, SIGNAL ( showSettings() ), this, SLOT ( showSettingsByPass() ) );
  connect ( object, SIGNAL ( showHelp( QString ) ), this, SLOT ( showHelpByPass( QString ) ) );
  connect ( object, SIGNAL ( showStatusWidget ( QWidget* ) ), this, SLOT ( showStatusWidgetByPass ( QWidget* ) ) );
  connect ( object, SIGNAL ( removeStatusWidget ( QWidget* ) ), this, SLOT ( removeStatusWidgetByPass ( QWidget* ) ) );

//   connect(this, SIGNAL(reloadSettings()), object, SLOT(reloadSettingsByPass()));
}

void MObject::settingsChanged() {}

void MObject::print() {}

void MObject::save() {}

void MObject::doExport() {}

void MObject::redo() {}

void MObject::undo() {}

void MObject::cut() {}

void MObject::copy() {}

void MObject::paste() {}

void MObject::reload() {}

void MObject::errorByPass ( const QString &string )
{
  emit error ( string );
}

void MObject::warningByPass ( const QString &string )
{
  emit warning ( string );
}

void MObject::informationByPass ( const QString &string )
{
  emit information ( string );
}

void MObject::addMenuByPass ( QMenu *newMenu, const QString &parent )
{
  emit addMenu ( newMenu, parent );
}

void MObject::delMenuByPass ( QMenu *del )
{
  emit delMenu ( del );
}

void MObject::addActionByPass ( QAction *newAction, const QString &parent )
{
  emit addAction ( newAction, parent );
}

void MObject::delActionByPass ( QAction *del )
{
  emit delAction ( del );
}

void MObject::addToolbarByPass ( QToolBar *newBar )
{
  emit addToolbar ( newBar );
}

void MObject::delToolbarByPass ( QToolBar *delBar )
{
  emit delToolbar ( delBar );
}

void MObject::addNewDockWidgetByPass ( QDockWidget *widget, const Qt::DockWidgetArea &area )
{
  emit addNewDockWidget ( widget, area );
}

void MObject::removeDockWidgetByPass ( QDockWidget *dockWidget )
{
  emit removeDockWidget ( dockWidget );
}

void MObject::setCentralWidgetByPass ( QWidget *centralWidget )
{
  emit setCentralWidget ( centralWidget );
}

void MObject::dbLoginByPass()
{
  emit dbLogin();
}

void MObject::showSettingsByPass()
{
  emit showSettings();
}

void MObject::reloadSettingsByPass()
{
  emit reloadSettings();
}

void MObject::showHelpByPass( QString file )
{
  emit showHelp(file);
}

void MObject::showStatusWidgetByPass ( QWidget *widget )
{
  emit showStatusWidget ( widget );
}

void MObject::removeStatusWidgetByPass ( QWidget *widget )
{
  emit removeStatusWidget ( widget );
}

#include "mobject.moc"
