/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "config.h"

#include "mpushbutton.h"

#include <QSettings>

MPushButton::MPushButton(QWidget* parent)
	: QPushButton(parent)
{
	init();
}

MPushButton::MPushButton(const QString& text, QWidget* parent)
	: QPushButton(text, parent)
{
	init();
}

MPushButton::MPushButton(const QIcon& icon, const QString& text, QWidget* parent)
	: QPushButton(text, parent)
{
	init();
	// must be used to strip out text
	setIcon(icon);
}

void MPushButton::setShowText(bool anytime)
{
	myShowText = anytime;
	if(anytime)
		setText(myText);
}

bool MPushButton::showText() const
{
	return myShowText;
}

void MPushButton::setIcon(const QIcon& icon)
{
	QPushButton::setIcon(icon);
	
	if(QPushButton::icon().pixmap(QPushButton::iconSize()).isNull())
		return;
	
	if(showText())
		return;
	
	QSettings settings;
	if(!settings.value("iconsAndText", false).toBool()){
		if(!text().isEmpty())
			myText = text();
		setText("");
		if(toolTip().isEmpty())
			setToolTip(myText);
	}
}

void MPushButton::setIcon(QString iconPath, QString standardIconName)
{
#if QT_VERSION_MINOR < 6
	setIcon(QIcon(iconPath));
#else
	setIcon(QIcon::fromTheme(standardIconName, QIcon(iconPath)));
#endif
}

void MPushButton::init()
{
	setShowText(false);
}

#include "mpushbutton.moc"
