/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef OPENFILEEXTERNAL_H
#define OPENFILEEXTERNAL_H

#include "mobject.h"
#include "libmacro.h"

class MimeTypeModel;
class MimeType;

#include <QFileInfo>

/**
  * trys to open a given file with an external application
  * it can detect the filetype with help of the extension
  * or the file type can be given.
  * In last case the file is opened with the related application
  * without checking, if it is a file of that type
  */
class EXPORT_DLL OpenFileExternal : public MObject{
  Q_OBJECT
  public:
    OpenFileExternal(QObject *parent = 0);
    ~OpenFileExternal();

    /**
      * returns a list of known MimeTypes
      */
    static MimeTypeModel* mimeTypes();

    void setFile(QString file, QString mimeType = "");
    void setFile(QFileInfo file, QString mimeType = "");

    bool openFile(QString file, QString mimeType = "");
    bool openFile(QFileInfo file, QString mimeType = "");
    bool open();

  private:
		QString myFile;
		MimeType *myType;
		MimeTypeModel *model;
		bool isStream;

    /**
      * trys to detect the mimetype autometically (or returns myType, if set)
      * right now it uses the suffix
      * @todo detect mimetype by opening it and reading the header of the file
      * @todo it should create a new one if it detects an unknown MimeType
      */
    MimeType *detectType();
    /**
      * Opens the SearchExternalApp dialog to let the user add other applications
      * and choose one of the listed applications
      */
    QString searchApp(MimeType* mimeType);
};
#endif // OPENFILEEXTERNAL_H
