/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef PERSON_H
#define PERSON_H

#include "libmacro.h"

#include <QObject>
#include <QString>

class EXPORT_DLL Person : public QObject
{
  Q_OBJECT
  public:
    Person(QObject *parent = 0);
    Person(unsigned int personId, QObject *parent = 0);
    Person(QString newName, QString newGivenName, QObject *parent = 0);
    virtual ~Person();
    
		virtual bool isValid();
    void setId(unsigned int personId, bool setPName = true);
    unsigned int id() const;
		/**
			* @attention if the whole name  (both name and givenname) of the person should be changed, use this function
			* @attention it takes care, that the givenName is set before the name, so that the correct person is search from database
			* @attention (Think of Thomas Mann, Heinrich Mann, Klaus Mann
			*/
		void setName(QString newName, QString givenName, bool setPId = true);
		/**
			* @attention if you intend to set the givenname to, use setName(name, givenName) (function above)
			*/
    void setName(QString newName, bool setPId = true);
    QString name() const;
    void setGivenName(QString newName);
    QString givenName() const;
		QString combine() const;
    void setMarkForRemovel( bool remove = true );
    bool markForRemovel() const;
		Person *copyPerson();
		
    bool operator== ( const Person &person ) const;
		bool operator!= ( const Person &person ) const;
    
	protected:
		void init(int personId = 0);
		
  private:
    
    QString myName, myGivenName;
    unsigned int myPersonId;
    bool shouldBeRemoved;
    
  signals:
    void changed(Person *person);
};

#endif // PERSON_H
