/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "querythread.h"

#include "queryworker.h"

#include <QTime>
#include <QCoreApplication>

QueryThread::QueryThread(QObject* parent) : QThread(parent)
{
	finishedStartup = false;
	connect(this, SIGNAL(started()), this, SLOT(setStarted()));
}

void QueryThread::run()
{
	// create worker in new thread
	worker = new QueryWorker();
	qRegisterMetaType< QList<QVariant> >("QList<QVariant>");
	qRegisterMetaType< QSqlRecord >("QSqlRecord");
	connect(this, SIGNAL(addToQueue(QString, QList<QVariant>)),
					worker, SLOT(addToQueue(QString, QList<QVariant>)));
	connect(this, SIGNAL(configureDB(QString,QString,QString,QString,QString)),
					worker, SLOT(configureDB(QString,QString,QString,QString,QString)));
	connect(this, SIGNAL(cancel()), worker, SLOT(cancel()));
	connect(worker, SIGNAL(result(QSqlRecord)), this, SLOT(resultByPass(QSqlRecord)));
	connect(worker, SIGNAL(setRange(int,int)), this, SLOT(setRangeByPass(int,int)));
	connect(worker, SIGNAL(setValue(int)), this, SLOT(setValueByPass(int)));
	connect(worker, SIGNAL(finished(bool)), this, SLOT(finishedByPass(bool)));
	
	// starting our own event loop (in our own thread)
	exec();
}

bool QueryThread::isStarted()
{
	return finishedStartup;
}

bool QueryThread::waitForStart()
{
	if(isStarted())
		return true;
	
	// wait for thread to be started
	QTime dieTime = QTime::currentTime().addSecs(2);
	// Userevents should be excluded see http://lists.trolltech.com/qt-interest/2007-01/thread00133-0.html
	while( QTime::currentTime() < dieTime ){
		QCoreApplication::processEvents(QEventLoop::ExcludeUserInputEvents, 100);
		if(isStarted())
			return true;
	}
	
	if(isStarted())
		return true;
	return false;
}

void QueryThread::execQuery(QString queryString, QList<QVariant> bindValues)
{
	emit addToQueue(queryString, bindValues);
}

void QueryThread::cancelQuery()
{
	if(isStarted())
		emit cancel();
}

void QueryThread::dbSetup(QString dbname, QString username, QString password, QString host, QString driver)
{
	emit configureDB(dbname, username, password, host, driver);
}

void QueryThread::resultByPass(QSqlRecord record)
{
	emit result(record);
}

void QueryThread::setRangeByPass(int min, int max)
{
	emit setRange(min, max);
}

void QueryThread::setValueByPass(int value)
{
	emit setValue(value);
}

void QueryThread::finishedByPass(bool status)
{
	emit finished(status);
}

void QueryThread::setStarted()
{
	finishedStartup = true;
}

#include "querythread.moc"
