/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef SEARCHDETAILSWIDGET_H
#define SEARCHDETAILSWIDGET_H

#include "libmacro.h"
#include <mwidget.h>

#include "sqlconstraint.h"

/**
	* The base class of the Widgets, which are used to 
	* display and choose material type specific search criterias
	*/
class EXPORT_DLL SearchDetailsWidget : public MWidget
{
	Q_OBJECT
	public:
		SearchDetailsWidget(QWidget *parent = 0, Qt::WindowFlags flags = 0);
		virtual ~SearchDetailsWidget();
		/**
			* the table or view which should be searched
			* @param supportPersons if set to true and personsSupported() is also true,
			* @param supportPersons a table should be returned, which allow to search person
			* @see personsSupported()
			*/
		virtual QString searchTable(bool supportPersons) = 0;
		/**
			* A list with SQL constraints which should be applied to the table
			*/
		virtual QList<SqlConstraint> constraints() = 0;
		/**
			* should return true, if the returned table by searchTable can be search for persons
			* if supportPersons is set to true
			* @see searchTable
			*/
		virtual bool personsSupported() = 0;
		
	protected slots:
		void startSearchByPass();
		
	signals:
		void startSearch();
};

#endif // SEARCHDETAILSWIDGET_H
