/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef SEARCHEXTERNALAPP_H
#define SEARCHEXTERNALAPP_H

#include "mdialog.h"
#include "libmacro.h"

class Program;

#include <QDir>

namespace Ui
{

  class SearchExternalApp;
}

/**
  * presents the user a dialog with a QListWidget where the user can choose
  * one of the apps from foundApps or he can add one manually
  */

class EXPORT_DLL SearchExternalApp : public MDialog
{
    Q_OBJECT
  public:
    /**
      *
      */
    SearchExternalApp ( QWidget *parent = 0 );
    ~SearchExternalApp();

    void init ( QList<Program*> findAppsList );
    /**
      * @return the selected application
      */
    QString selectedApp();
    /**
      * @return true if the user whishes to set the selectedApp() as the default app for this MimeType
      */
    bool setAsDefault();
    /**
      * @return a List of directories where applicatios are searched
      */
    static QList<QDir> defaultPaths();

  public slots:
    /**
      * if only one application is found, this one is used and the dialog is not shown at all
      * if more than one apps are found, the normal QDialog::exec() is called
      */
    int exec();

  private slots:
    /**
      * opens the QFileDialog and let the user add a new application
      */
    void searchApp();
    /**
      * sets the curApp to the selected one
      */
    void itemChanged ( QString currentText );

  private:
    Ui::SearchExternalApp *m_ui;
    QList<Program*> apps;
    QString curApp;

    /**
      * searches all application's with name of appNames at standard locations
      * @param appNames a QStringList with names of applications,
      * @param appNames which should be looked for at the places
      */
    QList<Program*> findApps ( QList<Program*> appNames );
};

#endif // SEARCHEXTERNALAPP_H
