/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "setupwizarddb.h"

#include "sqllogin.h"
#include "dbsettings.h"
#include "dbupdatewidget.h"
#include "sqlinit.h"

SetupWizardDB::SetupWizardDB ( QString newTitle, SqlInit *dbInit, QObject *parent ) : SetupWizardObject ( parent )
{
  needDBSettings = true;
  needDBUpdate = true;
  setTitle ( newTitle );
	sqlInit = dbInit;
	sqlInit->setParent(this);
}

bool SetupWizardDB::needed()
{
	if(!sqlInit)
		return false;

	connectByPass ( sqlInit );
	needDBSettings = true;
	needDBUpdate = true;
	if ( sqlInit->login(true) == SqlInit::LoggedIn ){
		needDBSettings = false;
		if(sqlInit->login(false) == SqlInit::NoUpdateNeeded)
			needDBUpdate = false;
  }
  // we should disconnect it here,
	// because DBUpdateWidget handels already all signals from sqlInit
	sqlInit->disconnect(this);
  if ( needDBSettings || needDBUpdate )
    return true;
  return false;
}

bool SetupWizardDB::finish ( SetupWizardWidget *finishWidget )
{
  if ( !finishWidget )
    return false;

  return finishWidget->finish();
}

QList<SetupWizardWidget*> SetupWizardDB::wizardWidgets()
{
  QList<SetupWizardWidget*> widgets;
  if ( needDBSettings ) {
    widgets.append ( new DBSettings ( this ) );
    connectByPass ( widgets[0] );
  }

  if ( needDBUpdate ) {
    widgets.append ( new DBUpdateWidget ( title(), sqlInit, this ) );
    connectByPass ( widgets[widgets.size() - 1] );
  }

  return widgets;
}

#include "setupwizarddb.moc"
