/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "shoppingcartdelegate.h"

#include "medium.h"
#include "materialpersonsmodel.h"
#include "materialperson.h"
#include "mediamodel.h"
#include "msqldatabase.h"

#include <QColor>
#include <QPainter>
#include <QSqlQuery>
#include <QSqlRecord>

ShoppingCartDelegate::ShoppingCartDelegate ( MediaModel *model, QWidget* parent ) : QStyledItemDelegate ( parent )
{
  this->model = model;
	paintWidget = new MediumWidget();
  
  MSqlDatabase db = MSqlDatabase::database();
  bool closeDB = !db.isOpen();

  if ( closeDB )
    db.open();

  if ( db.isOpen() ) {
    QSqlQuery query;
    query.exec ( "SELECT id, shortName FROM currencies" );

    while ( query.next() ) {
      mapCurrencies.insert ( query.record().value ( "id" ).toUInt(), query.record().value ( "shortName" ).toString() );
    }

    if ( closeDB )
      db.close();
  }
  
  setMode(MediumWidget::Shopping);
}

ShoppingCartDelegate::~ShoppingCartDelegate()
{
	delete paintWidget;
}

void ShoppingCartDelegate::setMode(MediumWidget::Mode mode)
{
	paintWidget->setMode(mode);
	myMode = mode;
}

QSize ShoppingCartDelegate::sizeHint ( const QStyleOptionViewItem& option, const QModelIndex& index ) const
{
  if ( !index.isValid() )
    return QSize();

  Medium *medium = static_cast<Medium*> ( index.internalPointer() );
	
	paintWidget->setMedium(medium);
	paintWidget->setBuyData(mapBuyData.value(medium), mapQuantity.value(medium));
	
  return paintWidget->sizeHint();
}

void ShoppingCartDelegate::paint ( QPainter* painter, const QStyleOptionViewItem& option, const QModelIndex& index ) const
{
  if ( !index.isValid() )
    return;

  Medium *medium = model->mediumFromIndex(index);

	paintWidget->setMedium(medium);
	paintWidget->setBuyData(mapBuyData.value(medium), mapQuantity.value(medium));
	paintWidget->paintMedium(painter, option);
}

void ShoppingCartDelegate::setMaxWidth ( unsigned int maxWidth )
{
  paintWidget->setMaxWidth(maxWidth);
}

void ShoppingCartDelegate::setQuantity ( QModelIndex index, int quantity )
{
  if(!index.isValid())
    return;
  
  setQuantity(model->mediumFromIndex(index), quantity);
}

void ShoppingCartDelegate::setQuantity ( Medium* medium, int quantity )
{
  if(!medium)
    return;
  
  if ( quantity < 0 ) {
    quantity = 1;

    if ( mapQuantity.contains ( medium ) )
      quantity = mapQuantity.value ( medium ) + 1;
  }

   mapQuantity.insert ( medium, quantity );
}

void ShoppingCartDelegate::setBuyData ( QModelIndex index, BuyData data )
{
  if ( !index.isValid() )
    return;

  setBuyData(model->mediumFromIndex(index), data);
}

void ShoppingCartDelegate::setBuyData ( Medium* medium, BuyData data )
{
  if(!medium)
    return;
  
  // always insert, because one item (key: row) can be checked by various shops
  mapBuyData.insert ( medium, data );
}

void ShoppingCartDelegate::clearBuyData()
{
  mapBuyData.clear();
}

QMap< Medium*, BuyData > ShoppingCartDelegate::buyDataList()
{
  return mapBuyData;
}

QMap< unsigned int, QString > ShoppingCartDelegate::mapCurrenciesList()
{
	return mapCurrencies;
}

void ShoppingCartDelegate::remove ( QModelIndex index )
{
  if(!index.isValid())
    return;
  
  remove(model->mediumFromIndex(index));
}

void ShoppingCartDelegate::remove ( Medium* medium )
{
  if(!medium)
    return;
  
  mapBuyData.remove ( medium );
  mapQuantity.remove ( medium );
}

#include "shoppingcartdelegate.moc"
