/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef SQLFIELD_H
#define SQLFIELD_H

#include "libmacro.h"

#include <QString>
#include <QVariant>
#include <QMultiHash>

class EXPORT_DLL SqlField
{
  public:
    enum type { noType, integer, text, varchar, boolean, date, time, tinyint, Double, timestamp, blob };

    SqlField();
    SqlField ( QString myName = "", type myType = SqlField::noType, bool notNull = false, bool autoIncrement = false );
    bool isValid() const;
    QString name() const;
    void name ( QString myName );
    bool isPrimaryKey() const;
    void isPrimaryKey ( bool status );
    bool autoIncrement() const;
    void autoIncrement ( bool status );
    bool notNull() const;
    void notNull ( bool status );
    QVariant defaultValue() const;
    void defaultValue ( QVariant value );
    SqlField::type dataType() const;
    void dataType ( SqlField::type myType );
    unsigned int length() const;
    void length ( unsigned int len );
    bool isUnsigned() const;
    void isUnsigned ( bool status );
    bool isBinary() const;
    void isBinary ( bool status );
    bool zeroFill() const;
    void zeroFill ( bool status );
    QString sqlFieldType() const;
		
		SqlField *copy() const;
		
		bool operator== (const SqlField &field) const;
		bool operator!= (const SqlField &field) const;

  private:
    QString fieldName;
    bool pk;
    bool ai;
    bool nn;
    QVariant fieldDefValue;
    SqlField::type fieldType;
    unsigned int fieldLength;
    bool binary;
    bool fieldUnsigned;
    bool zf;
    QMultiHash<int, QString> convFieldTypes;

    void init();
};

#endif // SQLFIELD_H
