/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "sqllogin.h"
#include "ui_sqllogin.h"

#include "msqldatabase.h"

#include <QSettings>
#include <QSqlError>

#ifdef KDE
	#include <KWallet/Wallet>
#endif

SqlLogin::SqlLogin ( QWidget *parent ) : MDialog ( parent ), m_ui ( new Ui::SqlLogin )
{
  init();
}

SqlLogin::SqlLogin ( QString title, QString newConnName, QWidget *parent ) : MDialog ( parent ), m_ui ( new Ui::SqlLogin )
{
  connName = newConnName;
  init();
	m_ui->connectionDetailsLabel->setText( title );
}

SqlLogin::~SqlLogin()
{
  close();
  delete m_ui;
}

QStringList SqlLogin::data()
{
  QStringList result;
  result.append ( m_ui->usernameEdit->text() );
  result.append ( m_ui->passwordEdit->text() );
  return result;
}

void SqlLogin::reloadSettingsByPass()
{
  loadAndCheckSettings();
  emit reloadSettings();
}

bool SqlLogin::loadAndCheckSettings(QString username, QString host, QString database,
																		QString driver, QString password)
{
  QSettings qSettings;
  qSettings.beginGroup ( "database" );
	if(!username.isEmpty()){
		this->username = username;
#ifdef KDE
		// try to set password from kwallet the user
		if(wallet)
			if(wallet->isOpen())
				wallet->readPassword(username, this->password);
#endif
	}
	m_ui->usernameEdit->setText(this->username);
	
	if(!password.isEmpty())
		this->password = password;
	m_ui->passwordEdit->setText(this->password);
	
	if(!host.isEmpty()){
		this->host = host;
	}
	else{
		this->host = qSettings.value ( "host" ).toString();
	}
	if(!driver.isEmpty()){
		this->driver = driver;
	}
	else{
		this->driver = qSettings.value ( "type" ).toString();
	}
	if(!database.isEmpty()){
		this->database = database;
	}
	else{
		this->database = qSettings.value ( "database" ).toString();
	}
	
	useSSL = qSettings.value("ssl", false).toBool();
  qSettings.endGroup();

  if ( ( this->driver.contains ( "SQLITE" ) ) && ( !this->database.isEmpty() ) ) {
    return true;
  }

  else if ( ( !this->host.isEmpty() ) || ( !this->driver.isEmpty() ) || ( !this->database.isEmpty() ) ) {
    return true;
  }

  return false;
}

bool SqlLogin::checkSettings()
{
  QSettings qSettings;
  qSettings.beginGroup ( "database" );
  QString myDriver = qSettings.value ( "type" ).toString();
  QString myHost = qSettings.value ( "host" ).toString();
  QString myDatabase = qSettings.value ( "database" ).toString();
  qSettings.endGroup();

  if ( ( myDriver.contains ( "SQLITE" ) ) && ( !myDatabase.isEmpty() ) ) {
    return true;
  }

  else if ( ( !myHost.isEmpty() ) || ( !myDriver.isEmpty() ) || ( !myDatabase.isEmpty() ) ) {
    return true;
  }

  return false;
}

int SqlLogin::exec()
{
  // try to get already established connections
  MSqlDatabase checkDB;

  if ( connName.isEmpty() ) {
    checkDB = MSqlDatabase::database();
  }

  else {
    checkDB = MSqlDatabase::database ( connName );
  }

  if ( checkDB.open() ){
    checkDB.close();
    return QDialog::Accepted;
  }

  // if we have database like SQLite where no login is needed, we doesn't show the
  // login screen, but connect and return accepted
  if ( driver.contains ( "SQLITE" ) ) {
    finish();
    // parts of the standard exec call of QDialog
    int res = result();
// 		bool destructiveClose = testWFlags(WDestructiveClose);
// 		if(destructiveClose)
//         delete this;

    if ( res == QDialog::Accepted ) {
      return res;
    }

    else {
      emit error ( m_ui->statusLabel->text() );
      return QDialog::Rejected;
    }
  }

	// try to finish
	if((!username.isEmpty()) && (!password.isEmpty())
		&& (!host.isEmpty()) && (!database.isEmpty())
		&& (!driver.isEmpty())){
		finish();
		int res = result();
		if ( res == QDialog::Accepted )
      return res;
	}
	
  return QDialog::exec();
}


void SqlLogin::changeEvent ( QEvent *e )
{
  QDialog::changeEvent ( e );

  switch ( e->type() ) {

    case QEvent::LanguageChange:
      m_ui->retranslateUi ( this );
      break;

    default:
      break;
  }
}

void SqlLogin::finish()
{
	username = m_ui->usernameEdit->text();
	password = m_ui->passwordEdit->text();
	
  if ( !driver.contains ( "SQLITE" ) ) {
    if ( username.isEmpty() ) {
      displayError ( tr ( "Please enter your username" ) );
      return;
    }

    if ( password.isEmpty() ) {
      displayError ( tr ( "Please enter your password" ) );
      return;
    }
  }

// could not be done, if we have things from command line
//   if ( !loadAndCheckSettings() ) {
//     emit showSettings();
//     return;
//   }

	MSqlDatabase db;
	if ( connName.isEmpty() ) {
    db = MSqlDatabase::database();
  }
  else {
    db = MSqlDatabase::database ( connName );
  }
  if ( !connName.isEmpty() ) {
    if ( db.connectionName() != connName ) {
      if ( db.connectionNames().contains ( connName ) ) {
        MSqlDatabase::removeDatabase ( connName );
      }

      db = MSqlDatabase::addDatabase ( driver, connName );
    }
  }

  else {
    if ( !db.isValid() ) {
      db = MSqlDatabase::addDatabase ( driver );
    }

    if ( db.driverName() != driver ) {
      MSqlDatabase::removeDatabase ( QLatin1String ( "defaultConnection" ) );
      db = MSqlDatabase::addDatabase ( driver );
    }
  }

	if(useSSL){
		emit error("using ssl");
		db.setConnectOptions("CLIENT_SSL=1");
	}
  db.setDatabaseName("");
  if(!withoutDatabase)
    db.setDatabaseName ( database );

  if ( !driver.contains ( "SQLITE" ) ) {
    db.setHostName ( host );
    db.setUserName ( username );
    db.setPassword ( password );
  }

  if ( !db.open() ) {
    if(!withoutDatabase){
      // may be the database does not exists,
      // but SqlInit creates it, so try to connect without database
      withoutDatabase = true;
      finish();
      return;
    }
    else{
			// both tries (with and wihtout database) failed,
			// so it must be a wrong username and password.
			// Therefore we set withoutDatabase back initial value (false)
			withoutDatabase = false;
		}
    // we already tried without database and it doesn't work, so we give up
    displayError ( tr ( "Can not connect to database:\n%1" ).arg ( db.lastError().text() ) );
		password = "";
    m_ui->passwordEdit->setText ( password );
    return;
  }

#ifdef KDE
	if((!username.isEmpty()) && (!password.isEmpty()))
		if(wallet)
			if(wallet->isOpen())
				wallet->writePassword(username, password);
#endif

  accept();
}

void SqlLogin::showSettingsDialog()
{
  emit showSettings();
}

void SqlLogin::usernameChanged()
{
	username = m_ui->usernameEdit->text();
	password = "";
	m_ui->passwordEdit->setText("");
#ifdef KDE
	// try to get password for user
	if(!wallet)
		return;
	if(!wallet->isOpen())
		return;
	QMap<QString, QString> list;
	wallet->readPasswordList("*", list);
	foreach(QString key, list.uniqueKeys()){
		if(key == username){
			password = list.value(key);
			m_ui->passwordEdit->setText(password);
		}
	}
#endif
}

void SqlLogin::init()
{
	useSSL = false;
  withoutDatabase = false;
  setWindowModality ( Qt::WindowModal );
  m_ui->setupUi ( this );
	
#ifdef KDE
	wallet = KWallet::Wallet::openWallet(KWallet::Wallet::NetworkWallet(), winId());
	if(wallet){
		QSettings settings;
		QString myFolder = settings.value("appNameShort", "Meganizer").toString();
		if(!wallet->hasFolder(myFolder)){
			wallet->createFolder(myFolder);
		}
		if(wallet->setFolder(myFolder) != 0){
			wallet->closeWallet(myFolder, true);
		}
	}
#endif

  connect ( m_ui->loginButton, SIGNAL ( clicked() ), this, SLOT ( finish() ) );
  connect ( m_ui->cancelButton, SIGNAL ( clicked() ), this, SLOT ( reject() ) );
  connect ( m_ui->settingsButton, SIGNAL ( clicked() ), this, SLOT ( showSettingsDialog() ) );
	connect ( m_ui->usernameEdit, SIGNAL (editingFinished()), this, SLOT ( usernameChanged()));

  if ( !loadAndCheckSettings() ) {
    emit showSettings();
  }
}

void SqlLogin::displayError ( QString msg )
{
  m_ui->statusLabel->setText ( QString ( "<font color='red'>%1</font>" ).arg ( msg ) );
}

void SqlLogin::displayInformation ( QString msg )
{
  m_ui->statusLabel->setText ( QString ( "<font color='Green'>%1</font>" ).arg ( msg ) );
}

#include "sqllogin.moc"
