/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "mediumownerwidgettest.h"

#include <medium.h>
#include <msqldatabase.h>
#include <mediumownerwidget.h>

#include <QtTest/QtTest>
#include <QTableView>
#include <QPushButton>
#include <QSqlQuery>
#include <QSqlRecord>
#include <QPoint>

#include <unistd.h>

void MediumOwnerWidgetTest::initTestCase()
{
	DBTestInit *testInit = new DBTestInit(this);
	if(!testInit->initTestDB()){
		QSKIP("Could not create test database", SkipAll);
	}
	
	keys = testInit->prepareMediumKeys("Testtype");
	if(!keys.valid)
		QSKIP("Could not prepare needed foreignkeys", SkipAll);
	medId = 0;
}

void MediumOwnerWidgetTest::testDBOpen()
{
	MediumOwnerWidget *medOwner = new MediumOwnerWidget();
	medOwner->setMedium(0);

	Medium *med = prepareMedium();
	if(!med){
		delete medOwner;
		QSKIP("Could not prepare Medium!", SkipSingle);
	}
	
	// check wether all work with closed db connection
	MSqlDatabase db = MSqlDatabase::database();
	db.close();
	medOwner->setMedium(med);
	QCOMPARE(db.isOpen(), false);
	QCOMPARE(medOwner->store(), true);
	QCOMPARE(db.isOpen(), false);
	
	medOwner->setMedium(0);
	
	// check wether all work with db open
	db.open();
	medOwner->setMedium(med);
	QCOMPARE(db.isOpen(), true);
	QCOMPARE(medOwner->store(), true);
	QCOMPARE(db.isOpen(), true);
	db.close();
	
	delete medOwner;
	delete med;
}

void MediumOwnerWidgetTest::changingOwner()
{
	MSqlDatabase db = MSqlDatabase::database();
	// add new user
	db.open();
	QSqlQuery query;
	QString name = "User";
	unsigned int newUserId = 0;
	if(!query.exec("INSERT INTO users (name, givenName) VALUES('"+ name +"', 'New')")){
		db.close();
		QSKIP("Could not create new owner", SkipSingle);
	}
	query.exec("SELECT id FROM users WHERE name='"+ name +"'");
	query.first();
	newUserId = query.record().value("id").toUInt();
	if(newUserId == 0){
		db.close();
		QSKIP("Could not create new owner", SkipSingle);
	}
	db.close();
	
	MediumOwnerWidget *medOwner = new MediumOwnerWidget();
	medOwner->show();
	medOwner->setMedium(0);

	Medium *med = prepareMedium();
	if(!med){
		delete medOwner;
		QSKIP("Could not prepare Medium!", SkipSingle);
	}
	
	// check wether all work with closed db connection
	db.close();
	medOwner->setMedium(med);
	
	// find needed gui elements
	QTableView *ownerView = medOwner->findChild<QTableView*>("ownerView");
	if(!ownerView){
		delete med;
		delete medOwner;
		QSKIP("Could not find owner view", SkipSingle);
	}
	QPushButton *addButton = medOwner->findChild<QPushButton*>("addButton");
	if(!addButton){
		delete med;
		delete medOwner;
		QSKIP("Could not find add button", SkipSingle);
	}
	QPushButton *removeButton = medOwner->findChild<QPushButton*>("removeButton");
	if(!removeButton){
		delete med;
		delete medOwner;
		QSKIP("Could not find remove button", SkipSingle);
	}
	QTableView *userView = medOwner->findChild<QTableView*>("userView");
	if(!userView){
		delete med;
		delete medOwner;
		QSKIP("could not find user view", SkipSingle);
	}
	
	// remove owner
	ownerView->selectRow(0);
	removeButton->setEnabled(true);
 	QTest::mouseClick(removeButton, Qt::LeftButton);
	QVERIFY(ownerView->model()->rowCount() == 0);
	
	// add new owner
	bool rowSelected = false;
	for(int i = 0; i < userView->model()->rowCount(); i++){
		if(userView->model()->data(userView->model()->index(i, 0)).toString() == "User"){
			userView->selectRow(i);
			rowSelected = true;
			break;
		}
	}
	QVERIFY(rowSelected);
	addButton->setEnabled(true);
 	QTest::mouseClick(addButton, Qt::LeftButton);
	QVERIFY(ownerView->model()->rowCount() == 1);
	
	// store changes
	QVERIFY(medOwner->store());
	unsigned int medId = med->id();
	delete med;
	med = new Medium();
	QVERIFY(med->select(medId, newUserId));
	QCOMPARE(med->user(), newUserId);
	
	delete medOwner;
	delete med;
}

Medium* MediumOwnerWidgetTest::prepareMedium()
{
	Medium *med = new Medium();
	if(medId == 0){
		med->setTitle("Testtitle");
		med->setType(keys.testTypeId);
		med->setGenre(keys.testGenreId);
		med->setManufacturer(keys.testManuId);
		med->setBoughtPriceCurrency(keys.testCurId);
		med->setListPriceCurrency(keys.testCurId);
		
		if(!med->store()){
			return 0;
		}
		medId = med->id();
	}
	else{
		med->select(medId);
	}
	return med;
}

QTEST_MAIN(MediumOwnerWidgetTest)

#include "mediumownerwidgettest.moc"
