/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "cdconfig.h"

#include "cd.h"

#include "cds.h"
#include "cdedit.h"
#include <materialpersonsmodel.h>
#include <materialperson.h>
#include "tracksmodel.h"
#include "track.h"
#include <msqldatabase.h>

#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlError>
#include <QVariant>
#include <QTextTable>
#include <QTextCursor>
#include <QProgressBar>

CD::CD ( Medium *newParent ) : Medium ( newParent )
{
	myName = tr ( "Music Device" );
  myGroup = "cd";
	
	myDeviceId = 0;
  myDiscs = 0;
  myTracks = new TracksModel ( 0, this );
	connect(myTracks, SIGNAL(error(QString)), this, SLOT(errorByPass(QString)));
	connect(myTracks, SIGNAL(warning(QString)), this, SLOT(warningByPass(QString)));
	connect(myTracks, SIGNAL(information(QString)), this, SLOT(informationByPass(QString)));
	myTracks->setAlbumMode(true);

	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if ( closeDB )
		db.open();
  if ( db.isOpen() ) {
    QSqlQuery query;
    query.exec ( QString ( "SELECT id FROM materialtypes WHERE name='%1'" ).arg ( name() ) );

    if ( query.first() )
      setType ( query.value ( 0 ).toUInt() );
		if(closeDB)
			db.close();
  }
}

CD::~CD()
{
	delete myTracks;
}

Medium* CD::newMedium()
{
	return new CD();
}

bool CD::select ( QSqlRecord record )
{
	MSqlDatabase db = MSqlDatabase::database();
  bool closeDB = false;
  if ( !db.isOpen() )
    closeDB = true;

  if ( !Medium::select ( record ) )
    return false;

  if ( !record.contains ( "discs" ) ) {
    if ( closeDB )
			if( !db.open() )
				return false;

    QSqlQuery replaceRecord;

    replaceRecord.exec
    (
      QString ( "SELECT * FROM %1_cds WHERE materialsId=%2" )
      .arg ( CDs::group() )
      .arg ( id() )
    );

    if ( !replaceRecord.first() ) {
      if ( closeDB )
        db.close();

      return false;
    }

    record = replaceRecord.record();
  }

  setDiscs ( record.value ( "discs" ).toUInt() );
	setDevice( record.value( "devicesId").toUInt() );
  setPublicationDate ( record.value ( "publicationDate" ).toDate() );

  if ( closeDB )
		if( !db.open() )
			return false;

  QSqlQuery query;

  query.exec ( QString ( "SELECT id FROM materialtypes WHERE name='%1'" ).arg ( myName ) );

  if ( !query.first() ) {
    if ( closeDB )
      db.close();

    return false;
  }

  setType ( query.value ( 0 ).toUInt() );
	
	if(!myTracks->loadTracks(this)){
		if(closeDB)
			db.close();
		return false;
	}

  if ( closeDB )
    db.close();

  return true;
}

bool CD::select ( unsigned int searchId, unsigned int userId )
{
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if ( closeDB ){
		if( !db.open() ){
			return false;
		}
	}

  QSqlQuery query ( QString ( "SELECT * FROM viewmaterials WHERE id=%1" ).arg ( searchId ) );

  if ( !query.first() ){
		if(closeDB)
			db.close();
		return false;
	}

  bool result = select ( query.record() );

	if(closeDB)
		db.close();

  return result;
}

bool CD::store ( bool addOwner, bool startTransaction )
{
	bool isInsert = false;
	if(id() == 0)
		isInsert = true;
	
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if ( closeDB )
		if( !db.open() ){
			emit error(tr("Could not connect to database!\nError: %1").arg(db.lastError().text()));
			return false;
		}

	bool transaction = false;
	if(startTransaction)
		transaction = transactionsSupported;
	if(transaction)
		db.transaction();
	
  if ( !Medium::store ( addOwner, false ) ){
		if(transaction)
			if(!db.rollback())
				emit error(tr("Rolling back failed!"));
		if ( closeDB )
        db.close();
    return false;
	}

  // id is needed and should be set by Medium::store()
  if ( id() == 0 ){
		if(transaction)
			if(!db.rollback())
				emit error(tr("Rolling back failed!"));
		if ( closeDB )
        db.close();
    return false;
	}

  QSqlQuery query;
	query.exec(QString("SELECT id FROM %1_cds WHERE materialsId=%2").arg(CDs::group()).arg(id()));
	bool isUpdate = false;
	if(query.first())
		isUpdate = true;
  if ( isUpdate ) {
    if
    (
      !query.prepare
      (
        QString
        (
          "UPDATE %1_cds SET"
          " discs=:discs, publicationDate=:publicationDate, devicesId=:devicesId"
          " WHERE materialsId=:materialsId"
        )
        .arg ( CDs::group() )
      )
    ) {
			if(transaction)
				if(!db.rollback())
					emit error(tr("Rolling back failed!"));
      if ( closeDB )
        db.close();
			emit error(tr("Could not prepare update CD!\nQuery:%1\nError:\n%2").arg(query.lastQuery()).arg(query.lastError().text()));
      return false;
    }

  } else {
    if
    (
      !query.prepare
      (
        QString
        (
          "INSERT INTO %1_cds "
          "(materialsId, discs, publicationDate, devicesId) "
          "VALUES(:materialsId, :discs, :publicationDate, :devicesId)"
        )
        .arg ( CDs::group() )
      )
    ) {
			if(transaction)
				if(!db.rollback())
					emit error(tr("Rolling back failed!"));
      if ( closeDB )
        db.close();
			emit error(tr("Could not prepare insert CD statement!\nQuery:%1\nError:\n%2").arg(query.lastQuery()).arg(query.lastError().text()));
      return false;
    }
  }

  query.bindValue ( ":materialsId", id() );
  query.bindValue ( ":discs", discs() );
  query.bindValue ( ":publicationDate", publicationDate().toString ( "yyyy-MM-dd" ) );
	query.bindValue ( ":devicesId", device() );

 if ( !query.exec() ) {
	 if(transaction)
			if(!db.rollback())
				emit error(tr("Rolling back failed!"));
    if ( closeDB )
      db.close();
		emit error(tr("Could not execute SQL statement!\nQuery:%1\nError:\n%2").arg(query.lastQuery()).arg(query.lastError().text()));
    return false;
  }
  
  if ( !myTracks->store(this, isInsert) ) {
		if(transaction)
			if(!db.rollback())
				emit error(tr("Rolling back failed!"));
    if ( closeDB )
      db.close();
		emit error(tr("Could not store tracks!"));
    return false;
  }

	if(transaction)
			if(!db.commit())
				emit error(tr("Could not store medium! Commit has failed!"));
  if ( closeDB )
    db.close();

  return true;
}

bool CD::setValue ( QString attribute, QVariant value )
{
 	if(compatibilityMode())
		return Medium::setValue(attribute, value);
	
	if(attribute == "label"){
		
		return true;
	}
	
	if(attribute == "pubDate"){
		setPublicationDate(value.toDate());
		return true;
	}

	if(attribute == "discs"){
		setDiscs(value.toUInt());
		return true;
	}
	
	if(attribute == "track"){
		TracksModel *trackModel = 0;;
		Track *track = 0;
		QStringList items = value.toStringList();
		if(items.size() != 4)
			return false;
		if(items[2].isEmpty())
			return false;
		
		trackModel = tracks();
		if(!trackModel)
			return false;

		track = qobject_cast<Track*>(trackModel->addEmptyMedium());
		if(!track)
			return false;
		track->setAlbumMode(true);
		track->setDisc(items[0].toUInt());
		track->setTrackNumber(items[1].toUInt());
		track->setTitle(items[2]);
		track->setLength(QTime::fromString(items[3], "hh:mm:ss"));
		
		// set default artist and label
		track->setManufacturer(manufacturer(), false);
		track->setManufacturer(manufacturerName(), false);
		MaterialPersonsModel *model = materialPersons();
		if(model){
			if(model->rowCount() == 1){
				MaterialPerson *person = qobject_cast<MaterialPerson*>(model->personFromIndex(model->index(0,0)));
				if(person){
					if((!person->name().contains(tr("Various"))) && (!person->givenName().contains(tr("Various")))){
						model = track->materialPersons();
						if(model){
							model->addPerson(person);
						}
					}
				}
			}
		}
		return true;
	}
	
	if(attribute == "device"){
		bool check = false;
		value.toUInt(&check);
		if(check){
			setDevice(value.toUInt());
			return true;
		}
		setDevice(value.toString());
		return true;
	}
	return Medium::setValue(attribute, value);
}

QVariant CD::value ( QString attribute, int role )
{
	// formatting should be the same for al media,
	// unless there are good reasons to do it in another way
	if ( role != Qt::DisplayRole || compatibilityMode())
    return Medium::value(attribute, role);
  
	if(attribute == "label")
		return manufacturerName();
	
	if(attribute == "pubDate")
		return publicationDate();
	
	if(attribute == "discs")
		return discs();
	
	if(attribute == "device")
		return deviceName();
	
  return Medium::value(attribute, role);
}

EditMedium *CD::editWidget(MWidget* parent)
{
  return new CDEdit ( parent );
}

void CD::setDiscs ( unsigned int numDiscs )
{
  myDiscs = numDiscs;
}

unsigned int CD::discs() const
{
  return myDiscs;
}

void CD::setDevice(unsigned int deviceId, bool setName)
{
	myDeviceId = deviceId;
	if(setName){
		setDevice("", false);
		if(deviceId > 0){
			MSqlDatabase db = MSqlDatabase::database();
			bool closeDB = !db.isOpen();
			if(closeDB)
				db.open();
			if(db.isOpen()){
				QSqlQuery query;
				query.exec(QString("SELECT name FROM "+ CDs::group() +"_devices WHERE id=%1").arg(deviceId));
				if(query.first()){
					setDevice(query.record().value("name").toString(), false);
				}
				if(closeDB)
					db.close();
			}
		}
	}
  emit changed(this);
}

void CD::setDevice(QString name, bool setId)
{
	myDeviceName = name;
	if(setId){
		setDevice(0, false);
		if(!name.isEmpty()){
			MSqlDatabase db = MSqlDatabase::database();
			bool closeDB = !db.isOpen();
			if(closeDB)
				db.open();
			if(db.isOpen()){
				QSqlQuery query;
				query.exec(QString("SELECT id FROM "+ CDs::group() +"_devices WHERE name='%1'").arg(name));
				if(query.first()){
					setDevice(query.record().value("id").toUInt(), false);
				}
				if(closeDB)
					db.close();
			}
		}
	}
  emit changed(this);
}

unsigned int CD::device() const
{
	return myDeviceId;
}

QString CD::deviceName() const
{
	return myDeviceName;
}

void CD::setPublicationDate ( QDate newDate )
{
  myPublicationDate = newDate;
}

QDate CD::publicationDate() const
{
  return myPublicationDate;
}

Medium* CD::copyMedium()
{
    CD *cd = qobject_cast<CD*>(Medium::copyMedium());
		if(!cd)
			return 0;
		
		cd->setDevice(device(), false);
		cd->setDevice(deviceName(), false);
		cd->setDiscs(discs());
		cd->setPublicationDate(publicationDate());
		return cd;
}

bool CD::exportMedium(QTextTable* table, Medium::ExportFormat format, bool titleOnly
){
	if(!table)
		return false;
	
	// collecting the data which should be inserted	
	QStringList values;
	if(titleOnly){
		values << attrs ( "title" ).translation;
		values << attrs ( "persons" ).translation;
		values << attrs ( "discs" ).translation;
		values << attrs ( "label" ).translation;
		values << attrs ( "min" ).translation;
		values << tr("Wish");
	}
	else{
		values << title();
		MaterialPersonsModel *model = materialPersons();
		if(model){
			QStringList authors;
			for(int i = 0; i < model->rowCount(); i++){
				authors.append
				(
					QString("%1, %2")
						.arg(model->data(model->index(i, 0)).toString())
						.arg(model->data(model->index(i, 1)).toString().left(1))
				);
			}
			values << authors.join(tr(" and "));
		}
		values << QString::number(discs());
		values << manufacturerName();
		values << minsCombined();
		if(wish())
			values << tr("Yes");
	}
	
	// insert the place at the table we need
	if(!titleOnly){
		// the first row is present at the table, because empty table can not be created
#if QT_VERSION_MINOR < 5
		table->insertRows(table->rows(), 1);
#else
		table->appendRows(1);
#endif
	}
	if(table->columns() < values.size()){
#if QT_VERSION_MINOR < 5
		table->insertColumns(table->columns(), values.size() - table->columns());
#else
		table->appendColumns(values.size() - table->columns());
#endif
	}
	int row = table->rows() - 1;
	bool result = true;
	for(int i = 0; i < values.size(); i++){
		QTextTableCell cell = table->cellAt(row, i);
		QTextCursor cursor = cell.firstCursorPosition();
		cursor.insertText(values[i]);
	}
	return result;
}

TracksModel *CD::tracks() const
{
  return myTracks;
}

bool CD::removeTrack ( QModelIndex index )
{
  if ( !myTracks )
    return false;

  Track *track = dynamic_cast<Track*> ( myTracks->mediumFromIndex ( index ) );

  if ( !track )
    return false;

  track->setMarkForRemovel();

  return true;
}

QList<ColKeyTrans> CD::supportedAttrs()
{
	if(compatibilityMode())
		return Medium::supportedAttrs();
		
  QList<ColKeyTrans> attrs;
  attrs << Medium::attrs("source", 0);
	attrs << Medium::attrs("title", 1);
  attrs << Medium::attrs("persons", 2);
	attrs << ColKeyTrans(3, "label", tr ( "Label" ));
  attrs << ColKeyTrans(4, "pubDate", tr ( "Pub. Date" ));
	attrs << ColKeyTrans(5, "discs", tr ( "Discs" ));
  attrs << Medium::attrs("min", 6);
	attrs << ColKeyTrans(7, "device", tr("Device"));
	attrs << ColKeyTrans(8, "track", tr( "Track" ));
	attrs << Medium::attrs("listPrice", 9);
	attrs << Medium::attrs("listPriceCurrency", 10);
	attrs << Medium::attrs("description", 11);
	attrs << Medium::attrs("owner", 12);
  return attrs;
}

#include "cd.moc"
