/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "cdconfig.h"

#include "cdedit.h"

#include "cd.h"
#include "cds.h"
#include "editgenres.h"
#include "editmanufacturers.h"
#include "materialpersonsmodel.h"
#include "materialperson.h"
#include "editmaterialpersons.h"
#include "delegateforeignkey.h"
#include "tracksmodel.h"
#include "addmedium.h"
#include "mpushbutton.h"
#include "searchbutton.h"
#include "msqldatabase.h"
#include "editdevices.h"
#include "tracksview.h"

#include <QSqlQueryModel>
#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlError>
#include <QDateEdit>
#include <QSpinBox>
#include <QComboBox>
#include <QTableView>
#include <QWidget>
#include <QHBoxLayout>
#include <QVBoxLayout>

CDEdit::CDEdit ( QWidget* parent, Qt::WindowFlags flags )
    : EditMedium ( parent, flags )
{
	tracksView = new TracksView(this);
	connectByPass(tracksView);
	lowerTabWidget->addTab(tracksView, tr("Tracks:"));
	
	QHBoxLayout *ml = new QHBoxLayout();
	ml->addWidget(prepareLabel(tr("Pub. Date:")));
	pubDateEdit = new QDateEdit(this);
	ml->addWidget(pubDateEdit);
	pubDateEdit->setDisplayFormat(tr("MM/yyyy"));
	insertLayout(ml, metaDataTab);
	
	ml = new QHBoxLayout();
	ml->addWidget(prepareLabel(tr("Discs:")));
	discsEdit = new QSpinBox(this);
	ml->addWidget(discsEdit);
	discsEdit->setRange(0, 99);
	insertLayout(ml, metaDataTab);
	
	ml = new QHBoxLayout();
	ml->addWidget(prepareLabel(tr("Device:")));
	deviceEdit = new QComboBox(this);
	ml->addWidget(deviceEdit);
	SearchButton *button = new SearchButton(this);
	ml->addWidget(button);
	connect(button, SIGNAL(clicked()), this, SLOT(newDevice()));
	button->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
	button->setMaximumHeight(30);
	insertLayout(ml, metaDataTab);
	deviceModel = new QSqlQueryModel(this);
	deviceEdit->setModel(deviceModel);
	
	manufacturerWidget->setLabelText(tr("Label:"));
}

void CDEdit::loadData(Medium* medium)
{
	editedCD = qobject_cast<CD*>(medium);
  if ( !editedCD ) 
		return;
	
	discsEdit->setValue ( editedCD->discs() );
	pubDateEdit->setDate ( editedCD->publicationDate() );
	
	tracksView->setModel(editedCD->tracks());

  MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		if(!db.open())
			return;

	QSqlQuery query;
	// fetching default roleId
	query.exec ( QString ( "SELECT id FROM materialpersonsroles WHERE name='%1'" ).arg(tr("Artist")));
	if(query.first())
		personsWidget->setDefaultRole(query.record().value("id").toUInt());
	
	deviceModel->setQuery("SELECT id, name FROM "+ CDs::group() +"_devices ORDER BY name");
	deviceEdit->setModelColumn(deviceModel->record().indexOf("name"));
	deviceEdit->setCurrentIndex(deviceEdit->findText(editedCD->deviceName()));
	if(deviceEdit->currentText().isEmpty())
		deviceEdit->setCurrentIndex(0);
	
	if(closeDB)
    db.close();
}

bool CDEdit::prepareSave(Medium* medium)
{
  if ( !editedCD ) 
		return false;

	editedCD->setPublicationDate( pubDateEdit->date() );
  editedCD->setDiscs ( discsEdit->value() );
	editedCD->setDevice ( deviceEdit->currentText() );
	
  return true;
}

void CDEdit::newDevice()
{
	EditDevices edit(editedCD->deviceName(), this);
	connectByPass(&edit);
	if(edit.exec() != QDialog::Accepted)
		return;
	
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if(closeDB)
		db.open();
	if(db.isOpen()){
		deviceModel->setQuery(deviceModel->query());
		if(closeDB)
			db.close();
	}
	editedCD->setDevice(edit.selectedName(), false);
	editedCD->setDevice(edit.selectedId(), false);
	deviceEdit->setCurrentIndex(deviceEdit->findText(editedCD->deviceName()));
}

#include "cdedit.moc"
