/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "cdsdb.h"

#include "cds.h"

#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlError>

CDsDB::CDsDB ( QObject *parent ) : SqlInit ( parent )
{
  init();
}

void CDsDB::scheme()
{
  currentVersion = 6;
  thisName = CDs::group();
  moduleLabel = CDs::myName();

	SqlTable *devices = addTable ( thisName +"_devices" );
	SqlField *field = devices->appendField( "name", SqlField::varchar, true );
	field->length(50);
	devices->addIndex(field, 10);
	field = devices->appendField ( "description", SqlField::varchar );
	field->length(500);
	
	QStringList values;
	values << "" << tr("CD") << tr("Compact Disc");
	devices->appendValues(values);
	values.clear();
	values << "" << tr("Record") << tr("Record Description");
	devices->appendValues(values);
	values.clear();
	values << "" << tr("DVD") << tr("Digital Versatile Disc");
	devices->appendValues(values);
	values.clear();
	
  SqlTable *table = addTable ( thisName + "_cds" );
  field = table->appendField ( "materialsId", SqlField::integer, true );
  field->isUnsigned ( true );
  table->addForeignKey ( field->name(), "materials", "id" );

  field = table->appendField ( "discs", SqlField::integer );
  field->isUnsigned ( true );

  field = table->appendField ( "publicationDate", SqlField::date );
	table->addForeignKey ( devices );

	table = addTable ( thisName + "_tracks" );

  field = table->appendField ( "materialsId", SqlField::integer, true );
  field->isUnsigned ( true );
  table->addForeignKey ( field->name(), "materials", "id" );

  field = table->appendField ( "length", SqlField::time );
	
  table = addTable ( "materialtypes" );
  table->appendField ( "name", SqlField::varchar );
  table->appendField ( "description", SqlField::varchar );

  QStringList insert;
  insert << "" << tr ( "Music Device" ) << tr ( "All medias where music is stored on" );
	table->appendValues ( insert );
	insert.clear();
	insert << "" << tr ( "Track" ) << tr ( "A track of an artist" );
	table->appendValues ( insert );
  
  table = addTable("materialpersonsroles");
  table->appendField("name", SqlField::varchar);
  table->appendField("description", SqlField::varchar);
  
  insert.clear();
  insert << "" << tr("Artist") << tr("The artist which performs an track");
  table->appendValues(insert);
	
	// A CD can have tracks
	table = addTable ( thisName + "_ondisc" );
	
	field = table->appendField ( "materialhasmaterialsId", SqlField::integer, true );
  field->isUnsigned ( true );
  table->addForeignKey ( field->name(), "materialhasmaterials", "id" );
	
	field = table->appendField ( "disc", SqlField::integer );
	field->isUnsigned( true );
	field->length(2);
	table->addIndex( field );
	
	field = table->appendField ( "number", SqlField::integer );
	field->isUnsigned( true );
	field->length(3);
	field->defaultValue(0);
	table->addIndex( field );
	
	// a track can have tracks
	table = addTable ( thisName +"_trackhastracks");
	field = table->appendField ( "materialhasmaterialsId", SqlField::integer, true );
  field->isUnsigned ( true );
  table->addForeignKey ( field->name(), "materialhasmaterials", "id" );
	
	field = table->appendField("number", SqlField::integer);
	field->isUnsigned(true);
	field->length(3);
	field->defaultValue(0);
	
	QStringList view;
  view.append ( thisName +"_viewcds" );
  view.append ( "SELECT  mats.*, cds.discs, cds.publicationDate, cds.devicesId"
                "  FROM viewmaterials mats "
                "  LEFT JOIN "+ thisName +"_cds cds ON mats.id=cds.materialsId");
  views.append ( view );
	view.clear();
  view.append ( thisName +"_viewtracks" );
  view.append ( "SELECT  mats.*, tracks.length"
                "  FROM viewmaterials mats "
                "  LEFT JOIN "+ thisName +"_tracks tracks ON mats.id=tracks.materialsId");
  views.append ( view );
	
	addUpdateRoutine(5, &CDsDB::updateFrom4To5);
}

bool CDsDB::updateFrom4To5(QString thisName, QList<SqlTable*> tables, QString& errorString)
{
	QSqlQuery query;

	// add the cd type to already present musicdevices
	// get old typeId
	query.prepare("SELECT id FROM materialtypes WHERE name=:name");
	query.bindValue(":name", tr ( "Audio CD"));
	query.exec();
	if(!query.first())
		return false;
	if(query.size() != 1)
		return false;
	unsigned int typeId = query.record().value("id").toUInt();
	if(typeId <= 0){
		errorString.append(tr("Could not find id of Audio CD\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}
	
	// get device id
	query.prepare("SELECT id FROM "+ thisName +"_devices WHERE name=:name");
	query.bindValue(":name", tr ( "CD"));
	query.exec();
	if(!query.first())
		return false;
	if(query.size() != 1)
		return false;
	unsigned int deviceId = query.record().value("id").toUInt();
	if(deviceId <= 0){
		errorString.append(tr("Could not find id of Music Device\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}
	
	query.prepare("SELECT id FROM materials WHERE materialtypesId=:materialtypesId");
	query.bindValue(":materialtypesId", typeId);
	query.exec();
	QSqlQuery insertQuery;
	while(query.next()){
		insertQuery.prepare("UPDATE "+ thisName +"_cds SET devicesId=:devicesId WHERE materialsId=:materialsId");
		insertQuery.bindValue(":devicesId", deviceId);
		insertQuery.bindValue(":materialsId", query.record().value("id").toUInt());
		if(!insertQuery.exec()){
			errorString.append(tr("Could not convert all MusicDevices to new structure!\n%1\n%2")
														.arg(query.lastError().text()).arg(query.lastQuery()));
			return false;
		}
	}
	
	// rename old type name and remove new one
	query.prepare("SELECT * FROM materialtypes WHERE name=:name");
	query.bindValue(":name", tr ( "Music Device" ));
	query.exec();
	if(!query.first())
		return false;
	if(query.size() != 1)
		return false;
	unsigned int newTypeId = query.record().value("id").toUInt();
	QString name = query.record().value("name").toString();
	QString description = query.record().value("description").toString();
	if(newTypeId <= 0){
		errorString.append(tr("Could not find id of type Music Device\n%1\n%2")
													.arg(query.lastError().text()).arg(query.lastQuery()));
		return false;
	}
	// rename old type
	query.prepare("UPDATE materialtypes SET name=:name, description=:description WHERE id=:id");
	query.bindValue(":name", name);
	query.bindValue(":description", description);
	query.bindValue(":id", typeId);
	if(!query.exec())
		return false;
	
	// remove new type
	query.prepare("DELETE FROM materialtypes WHERE id=:id");
	query.bindValue(":id", newTypeId);
	if(!query.exec())
		return false;
	
	return true;
}

#include "cdsdb.moc"
