/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "cdconfig.h"

#include "track.h"

#include "cds.h"
#include "trackedit.h"
#include <materialpersonsmodel.h>
#include <editmedium.h>
#include <msqldatabase.h>
#include "tracksmodel.h"

#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlError>
#include <QVariant>
#include <QTextTable>
#include <QTextCursor>

Track::Track ( Medium *newParent ) : Medium ( newParent )
{
	myName = tr ( "Track" );
  myGroup = "track";
	myTracks = 0;
	
	myAlbumMode = false;
	myCollectionMode = false;
	myDisc = 0;
	myTrackNumber = 0;
	
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if ( closeDB )
		!db.open();
	if ( db.isOpen() ) {
    QSqlQuery query;
    query.exec ( QString ( "SELECT id FROM materialtypes WHERE name='%1'" ).arg ( myName ) );

    if ( query.first() )
      setType ( query.value ( 0 ).toUInt() );
		
		if(closeDB)
			db.close();
  }
}

Medium* Track::newMedium()
{
	return new Track();
}

bool Track::select( QSqlRecord record)
{
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
  if ( closeDB )
		if( !db.open() )
			return false;
  
  if(!Medium::select(record)){
		if(closeDB)
			db.close();
    return false;
	}
  
	if(!record.contains("length")){
    if ( closeDB )
			if( !db.open() )
				return false;
      
    QSqlQuery replaceRecord;
    replaceRecord.exec
    (
      QString("SELECT * FROM %1_tracks WHERE materialsId=%2")
				.arg(CDs::group())
				.arg(id())
    );
    if(!replaceRecord.first()){
      if(closeDB)
				db.close();
      return false;
    }
    record = replaceRecord.record();
  }
  
  setLength ( record.value ( "length" ).toTime() );

	if ( closeDB )
		if( !db.open() )
			return false;
  
  QSqlQuery query;
  query.exec ( QString ( "SELECT id FROM materialtypes WHERE name='%1'" ).arg ( name() ) );

  if ( !query.first() ){
    if(closeDB)
      db.close();
    return false;
  }

  setType ( query.value ( 0 ).toUInt() );
	
	if(!tracks()->loadTracks(this)){
		if(closeDB)
			db.close();
		return false;
	}
	
  if(closeDB)
    db.close();
  return true;
}

bool Track::select ( unsigned int searchId, unsigned int userId )
{
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
  if ( closeDB )
		if( !db.open() )
			return false;

  QSqlQuery query;
	query.exec( QString ( "SELECT * FROM materials WHERE id=%1" ).arg( searchId ) );

  if ( !query.first() ){
		if(closeDB)
			db.close();
    return false;
	}

  bool result = select(query.record());

	if(closeDB)
		db.close();
  return result;
}

bool Track::store( bool addOwner, bool startTransaction)
{
	bool isInsert = false;
	if(id() == 0)
		isInsert = true;
	
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
  if ( closeDB ){
		if( !db.open() )
			return false;
	}
	
	bool transaction = false;
	if(startTransaction)
		transaction = transactionsSupported;
	if(transaction)
		db.transaction();
	
  if ( !Medium::store(addOwner, false) ){
		if(transaction)
			if(!db.rollback())
				emit error(tr("Rolling back failed!"));
		if(closeDB)
			db.close();
    return false;
	}

  // id is needed and should be set by Medium::store()
  if(id() == 0){
		if(transaction)
			if(!db.rollback())
				emit error(tr("Rolling back failed!"));
		if(closeDB)
			db.close();
    return false;
	}
  
	QSqlQuery query;
	query.exec(QString("SELECT id FROM %1_tracks WHERE materialsId=%2").arg(CDs::group()).arg(id()));
	bool isUpdate = false;
	if(query.first())
		isUpdate = true;
  if(isUpdate){
    if
    (
      !query.prepare
      (
				QString
				(
					"UPDATE %1_tracks SET "
					"length=:length "
					"WHERE materialsId=:materialsId"
				)
				.arg(CDs::group())
			)
		)
    {
			if(transaction)
				if(!db.rollback())
					emit error(tr("Rolling back failed!"));
			if(closeDB)
				db.close();
			emit error(tr("Could not prepare statement to update track's data at tracks table!\nQuery: %1\nError: %2").arg(query.lastQuery()).arg(query.lastError().text()));
      return false;
    }
  }
  else{
    if
    (
      !query.prepare
      (
				QString
				(
					"INSERT INTO %1_tracks "
					"(materialsId, length) "
					"VALUES(:materialsId, :length)"
				)
				.arg(CDs::group())
      )
    )
    {
			if(transaction)
				if(!db.rollback())
					emit error(tr("Rolling back failed!"));
			if(closeDB)
				db.close();
			emit error(tr("Could not prepare statement to insert track's data at tracks table!\nQuery: %1\nError: %2").arg(query.lastQuery()).arg(query.lastError().text()));
      return false;
    }
  }
  query.bindValue(":materialsId", id());
  query.bindValue(":length", length().toString("HH:mm:ss"));
  
  if(!query.exec()){
		if(transaction)
			if(!db.rollback())
				emit error(tr("Rolling back failed!"));
		if(closeDB)
			db.close();
		emit error(tr("Could not execute the query to modife tracks table!\nQuery: %1\nError: %2").arg(query.lastQuery()).arg(query.lastError().text()));
    return false;
  }
  
  if( myTracks){
		if ( !myTracks->store(this, isInsert) ) {
			if(transaction)
				if(!db.rollback())
					emit error(tr("Rolling back failed!"));
			if ( closeDB )
				db.close();
			emit error(tr("Could not store tracks!"));
			return false;
		}
	}
  
	if(transaction)
		if(!db.commit())
			emit error(tr("Could not store medium! Commit has failed!"));	
  if(closeDB)
		db.close();

	return true;
}

bool Track::setValue ( QString attribute, QVariant value )
{
	if(compatibilityMode())
		return Medium::setValue(attribute, value);
	
  if(attribute == "length"){
		setLength(value.toTime());
		return true;
	}
	if(!myAlbumMode && !myCollectionMode)
		return Medium::setValue(attribute, value);
	
	bool result = false;
	if(attribute == "nr"){
		setTrackNumber(value.toUInt(&result));
		return result;
	}

	if(!myAlbumMode)
		return Medium::setValue(attribute, value);
	
	if(attribute == "disc"){
		setDisc(value.toUInt(&result));
		return result;
	}

	return Medium::setValue(attribute, value);
}

QVariant Track::value ( QString attribute, int role )
{
	if ( role != Qt::DisplayRole || compatibilityMode())
		return Medium::value(attribute, role);
	
	if(attribute == "disc")
		return disc();
	if(attribute == "nr")
		return trackNumber();
	if(attribute == "length")
		return length().toString("HH:mm:ss");
  return Medium::value(attribute, role);
}

EditMedium *Track::editWidget(MWidget* parent )
{
  return new TrackEdit ( parent );
}

void Track::setLength ( QTime duration )
{
  myLength = duration;
}

QTime Track::length()
{
  return myLength;
}

Medium* Track::copyMedium()
{
    Track *track = qobject_cast<Track*>(Medium::copyMedium());
		if(!track)
			return 0;
		
		track->setLength(length());
		track->setAlbumMode(myAlbumMode);
		track->setDisc(disc());
		track->setTrackNumber(trackNumber());
		return track;
}

bool Track::exportMedium(QTextTable* table, Medium::ExportFormat format, bool titleOnly
){
	if(!table)
		return false;
	
	// collecting the data which should be inserted	
	QStringList values;
	if(titleOnly){
		values << attrs ( "title" ).translation;
		values << attrs ( "persons" ).translation;
		values << attrs ( "length" ).translation;
		values << tr("Wish");
	}
	else{
		values << title();
		MaterialPersonsModel *model = materialPersons();
		if(model){
			QStringList authors;
			for(int i = 0; i < model->rowCount(); i++){
				authors.append
				(
					QString("%1, %2")
						.arg(model->data(model->index(i, 0)).toString())
						.arg(model->data(model->index(i, 1)).toString().left(1))
				);
			}
			values << authors.join(tr(" and "));
		}
		values << length().toString(tr("hh:mm:ss"));
		if(wish())
			values << tr("Yes");
	}
	
	// insert the place at the table we need
	if(!titleOnly){
		// the first row is present at the table, because empty table can not be created
#if QT_VERSION_MINOR < 5
		table->insertRows(table->rows(), 1);
#else
		table->appendRows(1);
#endif
	}
	if(table->columns() < values.size()){
#if QT_VERSION_MINOR < 5
		table->insertColumns(table->columns(), values.size() - table->columns());
#else
		table->appendColumns(values.size() - table->columns());
#endif
	}
	int row = table->rows() - 1;
	bool result = true;
	for(int i = 0; i < values.size(); i++){
		QTextTableCell cell = table->cellAt(row, i);
		QTextCursor cursor = cell.firstCursorPosition();
		cursor.insertText(values[i]);
	}
	return result;
}

bool Track::prepareTrack()
{
	if(id() > 0)
		return true;
	
	if(title().isEmpty())
		return false;
	
	// try to fetch all stuff withhelp of the title
	MSqlDatabase db = MSqlDatabase::database();
	bool closeDB = !db.isOpen();
	if ( closeDB ){
		if( !db.open() )
			return false;
	}
	
	QSqlQuery query;
	query.prepare("SELECT * FROM materials WHERE materialtypesId=:materialtypesId AND title=:title");
	query.bindValue(":materialtypesId", type());
	query.bindValue(":title", title());
	query.exec();
	if(query.size() == 1){
		if(query.first()){
			// fetch all needed stuff
			select(query.record().value("id").toUInt());
		}
	}
	if(closeDB)
		db.close();
	
	// check wether it is valid now
	if((id() > 0) && (genre() > 0) && (manufacturer() > 0)){
		if(closeDB)
			db.close();
		return true;
	}
	
	EditMedium *trackEdit = editWidget();
	trackEdit->setAddMode(this, false);
	if(trackEdit->exec() != QDialog::Accepted){
		if(closeDB)
			db.close();
		return false;
	}

	if((id() > 0) && (genre() > 0) && (manufacturer() > 0)){
		if(closeDB)
			db.close();
		return true;
	}

	if(myAlbumMode || myCollectionMode){
		// create the track now, because it doesn't exists
		if ( closeDB )
			if( !db.open() )
				return false;
		
		// we call it with no transactions, because if transactions
		// are supported, the caller started one already
		if(!store(true, false)){

			if(closeDB)
				db.close();
			return false;
		}

		if(closeDB)
			db.close();
		return true;
	}
	
	return false;
}

void Track::setDisc ( unsigned int num )
{
	myDisc = num;
}

unsigned int Track::disc()
{
	return myDisc;
}

void Track::setTrackNumber ( unsigned int num )
{
	myTrackNumber = num;
}

unsigned int Track::trackNumber()
{
 return myTrackNumber;
}

void Track::setAlbumMode ( bool enable )
{
	myAlbumMode = enable;
	if(myAlbumMode)
		setCollectionMode(false);
	emit changed(this);
}

bool Track::albumMode() const
{
	return myAlbumMode;
}

void Track::setCollectionMode(bool enable)
{
	myCollectionMode = enable;
	if(myCollectionMode)
		setAlbumMode(false);
	emit changed(this);
}

bool Track::collectionMode() const
{
	return myCollectionMode;
}

TracksModel* Track::tracks()
{
	if(!myTracks){
		myTracks = new TracksModel(0, this);
		myTracks->setCollectionMode(true);
		connect(myTracks, SIGNAL(error(QString)), this, SLOT(errorByPass(QString)));
		connect(myTracks, SIGNAL(warning(QString)), this, SLOT(warningByPass(QString)));
		connect(myTracks, SIGNAL(information(QString)), this, SLOT(informationByPass(QString)));
	}
	return myTracks;
}

bool Track::removeTrack(QModelIndex index)
{
  if ( !myTracks )
    return false;

  Track *track = dynamic_cast<Track*> ( myTracks->mediumFromIndex ( index ) );

  if ( !track )
    return false;

  track->setMarkForRemovel();

  return true;
}

QList<ColKeyTrans> Track::supportedAttrs()
{
	if(compatibilityMode())
		return Medium::supportedAttrs();
		
  QList<ColKeyTrans> attrs;
	if(myAlbumMode){
		attrs << ColKeyTrans(0, "disc", tr ( "Disc" ));
		attrs << ColKeyTrans(1, "nr", tr ( "Nr" ));
		attrs << Medium::attrs("persons", 2);
		attrs << Medium::attrs("title", 3);
		attrs << ColKeyTrans(4, "length", tr ( "Length" ));
	}
	else if(myCollectionMode){
		attrs << ColKeyTrans(0, "nr", tr ( "Nr" ));
		attrs << Medium::attrs("persons", 1);
		attrs << Medium::attrs("title", 2);
		attrs << ColKeyTrans(3, "length", tr ( "Length" ));
	}
	else{
		attrs << Medium::attrs("source", 0);
		attrs << Medium::attrs("title", 1);
		attrs << Medium::attrs("persons", 2);
		attrs << ColKeyTrans(3, "length", tr ( "Length" ));
		attrs << Medium::attrs("owner", 4);
	}
  return attrs;
}

#include "track.moc"
