/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "tracksview.h"

#include <elistaddbutton.h>
#include <elistremovebutton.h>
#include "tracksmodel.h"
#include <addmedium.h>
#include "etableview.h"

#include <QVBoxLayout>

TracksView::TracksView(QWidget* parent, Qt::WindowFlags flags): MWidget(parent, flags)
{
	model = 0;
	
	QHBoxLayout *ml = new QHBoxLayout(this);
	setLayout(ml);
	view = new ETableView(this);
	ml->addWidget(view);
	connect(view, SIGNAL(clicked(QModelIndex)), this, SLOT(selectTrack(QModelIndex)));
	
	QVBoxLayout *bl = new QVBoxLayout();
	ml->addLayout(bl);
	
	addTrackButton = new EListAddButton(this);
	bl->addWidget(addTrackButton);
	connect ( addTrackButton, SIGNAL(clicked()), this, SLOT(addTrack()));
	
	delTrackButton = new EListRemoveButton(this);
	bl->addWidget(delTrackButton);
	connect ( delTrackButton, SIGNAL(clicked()), this, SLOT(removeTrack()));
	bl->addStretch();
	
	selectTrack(QModelIndex());
}

void TracksView::setModel(TracksModel* model)
{
	selectTrack(QModelIndex());
	view->setModel(0);
	this->model = model;
	if(!model){
		selectTrack(QModelIndex());
		return;
	}

// 	setModel(model);
	model->configureView(view);
	view->setSelectionBehavior(QTableView::SelectItems);
	view->resizeColumnsToContents();
	selectTrack(QModelIndex());
}

void TracksView::addTrack(unsigned int trackId)
{
  if(!model)
    return;

	model->addMedium(trackId);
}

void TracksView::addTrack()
{
  if(!model)
    return;

  AddMedium addTrack(model, this);
  connectByPass(&addTrack);
  addTrack.exec();
}

void TracksView::removeTrack()
{
	if(!index.isValid())
		return;
	Medium *med = model->mediumFromIndex(index);
	if(!med){
		emit error(tr("Could not remove track!"));
		return;
	}
	med->setMarkForRemovel();
}

void TracksView::selectTrack(QModelIndex index)
{
	this->index = index;
	addTrackButton->setDisabled(true);
	delTrackButton->setDisabled(true);
	if(model){
		addTrackButton->setEnabled(true);
		if(index.isValid())
			delTrackButton->setEnabled(true);
	}
}

#include "tracksview.moc"
