/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "meganizerconfig.h"

#include "helpcenter.h"

#include <QProcess>
#include <QSettings>
#include <QLibraryInfo>
#include <QDir>
#include <QFileInfo>
#include <QLocale>

HelpCenter::HelpCenter ( QObject* parent ) : MObject ( parent )
{
	assistantProcess = 0;
}

HelpCenter::~HelpCenter()
{
	if (assistantProcess && assistantProcess->state() == QProcess::Running) {
		assistantProcess->terminate();
		assistantProcess->waitForFinished(3000);
	}
	delete assistantProcess;
}

void HelpCenter::showDocumentation ( QString page )
{
	if (!startAssistant())
		return;

	QString cmd;
	// check wether a keyowrd is used
	if(page.contains("keyword:", Qt::CaseInsensitive)){
		cmd = QString("activateKeyword %1").arg(page.right(page.length() - page.indexOf(":") - 1));
	}
	else{
		if(page.isEmpty())
			page = "meganizer/index.html";
		
		// prepend /
		if(page.left(1) != "/")
			page = "/"+ page;

		QSettings settings;
		QString cmd = QString("setSource %1%2")
										.arg(QString("qthelp://de.%1.%2")
														.arg(settings.value("prodName").toString().toLower())
														.arg(settings.value("appNameShort").toString().toLower())
										)
										.arg(page);
	}

	assistantProcess->write(cmd.toLocal8Bit() +'\0');
	
	// make sure, that the toc points to the same document as the content
	assistantProcess->write(QString("SyncContents").toLocal8Bit() + '\0');
}

bool HelpCenter::startAssistant()
{
	if(!assistantProcess)
		assistantProcess = new QProcess(this);
	
	if (assistantProcess->state() == QProcess::Running)
		return true;

	// find correct language file
	QSettings settings;
	QLocale qLocale;
	QString lang = settings.value (
		"defaultLang",
		qLocale.name().left(2).toUpper()
	).toString();
	QFileInfo docFile(QString(DOCDIR) + "/meganizer.qhc");
	if(!lang.isEmpty()){
		QFileInfo docLang(QString(DOCDIR) +"/meganizer_"+ lang.toLower() +".qhc");
		if(docLang.isReadable())
			docFile = docLang;
	}
	if(!docFile.isReadable()){
		// try to open an existing one
		QDir dir ( QString ( DOCDIR ) );

		if ( dir.exists() ) {
			QStringList filter;
			filter.append ( "*.qhc" );
			QStringList docFiles = dir.entryList ( filter, QDir::Files | QDir::Readable );
			if(docFiles.size() < 1){
					emit error(tr("Could not open documentation!\n"
												"Please install files to %1!")
												.arg(QString(DOCDIR)));
					return false;
			}
			else if(docFiles.size() == 1){
				docFile.setFile(dir, docFiles[0]);
				if(!docFile.isReadable()){
						emit error(tr("Could not open documentation!\n"
													"File %1 is not readable!")
												.arg(docFile.absoluteFilePath()));
						return false;
				}
				}
			else{
					emit error(tr("Could not open documentation!\n"
												"Please select a default language at the Settingspage!"));

					return false;
			}
	}
	}
	
 #if defined(Q_OS_WIN32)
	QString app = QLatin1String("assistant");
 #else
	QString app  = QLibraryInfo::location(QLibraryInfo::BinariesPath) + QDir::separator();
	#if defined(Q_OS_MAC)
		app += QLatin1String("Assistant.app/Contents/MacOS/Assistant");
	#else
		app += QLatin1String("assistant");
	#endif
 #endif

	QStringList args;
	args << QLatin1String("-collectionFile") << docFile.absoluteFilePath().toLatin1();
	args << QLatin1String("-enableRemoteControl");
	assistantProcess->start(app, args);
	if (!assistantProcess->waitForStarted()) {
			emit error(tr("Could not start Qt Assistant from %1.").arg(app));
			return false;
	}
	
	// expanding the tree
// 	assistantProcess->write(QString("ExpandToc 2").toLocal8Bit() + '\0');
	
	return true;
}

#include "helpcenter.moc"
