/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#ifndef MedienDB_H
#define MedienDB_H

#include <QMainWindow>

#include "ui_mediendb.h"

class MediaPlugin;
class SearchPlugin;
class ModuleListWidgetItem;
class ShoppingCart;
class HelpCenter;

#include <QPointer>

class QDockWidget;

class MedienDB : public QMainWindow, private Ui::MainWindow
{
    Q_OBJECT
  public:
    MedienDB ( int & argc, char ** argv, QWidget * parent = 0, Qt::WindowFlags flags = 0 );
    ~MedienDB();
		bool shouldExit();
		void init();
		
	protected:
		void closeEvent(QCloseEvent *event);

  private slots:
    bool loadPlugins();
    bool loadPlugin ( ModuleListWidgetItem* plugin, bool emitSetupWizard = false );
    bool loadPlugin ( QString file, bool emitSetupWizard = false );
    bool unloadPlugin ( ModuleListWidgetItem* plugin );
    bool unloadPlugin ( QObject *plugin );
    bool unloadPlugins();
    bool reloadPlugins();
    void showSetupWizard(bool msgOnNotNeeded = false);

		void fileProjects();
		void fileGenres();
		void fileManufacturers();
		void fileMaterialPersons();
		void fileRoles();
		void fileUser();
		void fileMinTypes();
    void fileExit();
		void editSetupWizard();
		void editDBBackup();
		void editSettings();
		void viewSummary();
    void helpHelp();
		void helpHelp(QString file);
    void helpMedienDB();
    void helpUsedSoftware();
    void helpAboutPlugins();

    void error ( QString msg );
    void warning ( QString msg );
    void information ( QString msg );

    void addNewDockWidget ( QDockWidget *widget, Qt::DockWidgetArea area );
    void newCentralWidget ( QWidget *widget );
    void newMenu ( QMenu *newMenu, QString parent );
    void newAction ( QAction *newAction, QString parent );
		void newToolbar( QToolBar *newBar );
		/**
			* @note deletion of the toolbar object is enough to remove the toolbar from the mainwindow
			*/
		void delToolbar( QToolBar *delBar );
		
		void showStatusWidget(QWidget *widget);
		/**
			* remove widget from statusbar
			*/
		void removeStatusWidget(QWidget *widget);

  private:
    void connectMSlots ( QObject *object );
		void processCommandlineArgs(int & argc, char ** argv);
		void showHelp();

    QString shortName;
    QList<MediaPlugin*> mediaPlugins;
    QList<SearchPlugin*> searchPlugins;
    QList<ModuleListWidgetItem*> pluginProtocol;
		QList<QPointer<QDockWidget> > dockWidgets;
		ShoppingCart *shoppingCart;
		HelpCenter *helpCenter;
		QString username, password, host, database, driver;
		bool doExit;

  signals:
    void checkSetupWizard();
};

#endif // MedienDB_H
