/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#include "modulestatedialog.h"

#include "modulestate.h"
#include <modulelistwidgetitem.h>
#include <minvalidator.h>
#include <mpushbutton.h>

#include <QTabWidget>
#include <QVBoxLayout>
#include <QHBoxLayout>

ModuleStateDialog::ModuleStateDialog(QList< ModuleListWidgetItem* > newMediaAndSearchProtocol, QWidget* parent, Qt::WindowFlags flags)
	: MDialog(parent, flags)
{
	QVBoxLayout *ml = new QVBoxLayout(this);
	setLayout(ml);
	
	QTabWidget *tabW = new QTabWidget(this);
	ml->addWidget(tabW);
	
	ModuleState *mState = new ModuleState(newMediaAndSearchProtocol, tabW);
	tabW->addTab(mState, tr("Media- and Searchplugins"));
	connectByPass(mState);
	connect(mState, SIGNAL(load(ModuleListWidgetItem*,bool)), this, SLOT(loadByPass(ModuleListWidgetItem*,bool)));
	connect(mState, SIGNAL(unload(ModuleListWidgetItem*)), this, SLOT(unloadByPass(ModuleListWidgetItem*)));
	connect(mState, SIGNAL(reloadPlugins()), this, SLOT(reloadPluginsByPass()));
	
	MINValidator *validator = new MINValidator(this);
	connectByPass(validator);
	mState = new ModuleState(validator->loadProtocol(), tabW);
	mState->setViewOnly();
	connectByPass(mState);
	tabW->addTab(mState, tr("MIN Validators"));
	
	QHBoxLayout *bl = new QHBoxLayout();
	ml->addLayout(bl);
	
	MPushButton *button = new MPushButton(this);
	bl->addWidget(button);
	setHelp("keyword:Plugins", button);
	
	bl->addStretch();
	
	button = new MPushButton(tr("Close"), this);
	bl->addWidget(button);
	connect(button, SIGNAL(clicked()), this, SLOT(accept()));
}

void ModuleStateDialog::reloadPluginsByPass()
{
	emit reloadPlugins();
}

void ModuleStateDialog::loadByPass(ModuleListWidgetItem* item, bool emitSetupWizard)
{
	emit load(item, emitSetupWizard);
}

void ModuleStateDialog::unloadByPass(ModuleListWidgetItem* item)
{
	emit unload(item);
}

#include "modulestatedialog.moc"
