/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/


#ifndef SETUPWIZARD_H
#define SETUPWIZARD_H

#include <mdialog.h>
#include "ui_setupwizard.h"

#include "setupwizardobject.h"

class MediaPlugin;
class SearchPlugin;

/**
  * the setup wizard guides the user to the configuration steps at the first start.
  * It creates the database and shows all needed settings.
  * Updates are also handled with this wizard
  *
  * @todo Login after DBSettings and create the database structure
  */

class SetupWizard: public MDialog, private Ui::SetupWizard
{
    Q_OBJECT
  public:
    SetupWizard ( QList<MediaPlugin*> newMediaPlugins, 
									QList<SearchPlugin*> newSearchPlugins, 
									QString username, QString host, 
									QString database, QString driver, QString password,
									QWidget *parent = 0, Qt::WindowFlags fl = 0);
    ~SetupWizard();
    bool isNeeded();
		void reload();

  public slots:
    int exec();

  protected:
    void closeEvent ( QCloseEvent *event );

  private slots:
		void setProgress();
		/**
			* @todo handling of reloads and related with it the error handling of finish()
			*/
    void next();
    void back();
		void markReload();

    void showErrorMessage ( QString msg );

  private:
    void loadObjects();
    void addObjectsWidgets ( SetupWizardObject *object );
		void unloadObjects();

		QString firstTitle;
    QString shortName;
    bool needed, doReload;
    QList<MediaPlugin*> mediaPlugins;
    QList<SearchPlugin*> searchPlugins;
		QString username, host, database, driver, password;

};

#endif // SETUPWIZARD_H
