/*
    Meganizer - Media Organzier is a media management software
    Copyright (C) 2010  EngSaS - Engineering Solutions and Services Langenbach. All rights reserved.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
#include "configamazon.h"
#include "connectmaterialtype.h"

#include <mediamodel.h>
#include <searchplugin.h>
#include <mediaplugin.h>

#include <QDir>
#include <QPluginLoader>
#include <QSettings>

ConnectMaterialType::ConnectMaterialType(Medium *medium, SearchPlugin* searchPlugin, QWidget* parent,	Qt::WindowFlags flags)
	: SettingsWidget(parent, flags)
{
	// setting up ui
	setupUi(this);
	
	setPlugins(medium, searchPlugin);
}

ConnectMaterialType::ConnectMaterialType(QWidget* parent, Qt::WindowFlags flags): SettingsWidget(parent, flags)
{
	setupUi(this);
}


void ConnectMaterialType::setPlugins(QString mediumGroup, SearchPlugin* searchPlugin)
{
	this->searchPlugin = searchPlugin;
	if(!searchPlugin)
		return;
	
	this->mediumGroup = mediumGroup;
	
	typeEdit->clear();
	typeLabel->setText
	(
		tr("Please select a category of %1, which should be searched for material type %2:")
			.arg(searchPlugin->name())
			.arg(name())
	);
	mapTypes = searchPlugin->supportedCategories();
	typeEdit->addItems(mapTypes.values());
	
	QSettings settings;
	settings.beginGroup(searchPlugin->settingsGroup());
	QString stored = settings.value(mediumGroup, QString()).toString();
	typeEdit->setCurrentIndex(typeEdit->findText(mapTypes.value(stored)));
	settings.endGroup();
}

void ConnectMaterialType::setPlugins(Medium* medium, SearchPlugin* searchPlugin)
{
	if(!medium)
		return;
	
	QString group = medium->group();
	// this means, it is a common Medium,
	// but it may know its type, if it is from database
	if(group.isEmpty() && medium->type() > 0){
		QSettings settings;
		QString pluginDir = settings.value ( "pluginDir" ).toString();
		if ( pluginDir.isEmpty() ) {
			pluginDir = QString ( PLUGINDIR );
		}
		//load and test plugin
		QDir dir ( pluginDir );
		if ( dir.exists() ) {
			QStringList files = dir.entryList ( QDir::Files );
			if ( files.size() > 0) {
				foreach ( QString file, files ) {
					QPluginLoader loader ( dir.filePath ( file ) );
					QObject *plugin = loader.instance();
					if ( plugin ) {
						MediaPlugin *newPlugin = qobject_cast<MediaPlugin *> ( plugin );
						if ( newPlugin ) {
							if(newPlugin->supportedMaterialTypes().contains(medium->typeName())){
								MediaModel *model = newPlugin->newModel(medium->typeName());
								Medium *med = model->newMedium();
								group = med->group();
								delete med;
								delete model;
								delete plugin;
								break;
							}
							delete plugin;
						}
					}
				}
			}
		}
	}
	
	setPlugins(group, searchPlugin);
}

QString ConnectMaterialType::category()
{
	return mapTypes.key(typeEdit->currentText());
}

bool ConnectMaterialType::loadSettings()
{
	if(!searchPlugin)
		return false;
	
	if(mediumGroup.isEmpty())
		return false;
	
	bool result = false;
	QSettings settings;
	settings.beginGroup(searchPlugin->settingsGroup());
	QString stored = settings.value(mediumGroup, QString()).toString();
	if(typeEdit->findText(mapTypes.value(stored)) > -1)
		result = true;
	typeEdit->setCurrentIndex(typeEdit->findText(mapTypes.value(stored)));
	settings.endGroup();
	
	return result;
}

bool ConnectMaterialType::saveSettings()
{
	if(!searchPlugin)
		return false;
	
	if(mediumGroup.isEmpty())
		return false;
	
	QSettings settings;
	settings.beginGroup(searchPlugin->settingsGroup());
	settings.setValue(mediumGroup, mapTypes.key(typeEdit->currentText()));
	settings.endGroup();

	return true;
}

#include "connectmaterialtype.moc"
